/*****************************************************************************
 *
 * COMMANDS.C - External command functions for NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   01-17-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"

extern char	log_file[MAX_FILENAME_LENGTH];
extern char     status_file[MAX_FILENAME_LENGTH];
extern char     command_file[MAX_FILENAME_LENGTH];
extern char     temp_file[MAX_FILENAME_LENGTH];
extern char     comment_file[MAX_FILENAME_LENGTH];

extern int      sigshutdown;
extern int      sigrestart;

extern int      current_comment_id;

extern int      check_external_commands;

extern time_t   last_command_check;

extern timed_event      *event_list_high;
extern timed_event      *event_list_low;
extern service          *service_list;
extern host             *host_list;


/******************************************************************/
/****************** EXTERNAL COMMAND PROCESSING *******************/
/******************************************************************/

/* checks for the existence of the external command file and processes all
   commands found in it */
void check_for_external_commands(void){
	FILE *fp;
	char buffer[MAX_INPUT_BUFFER];
	char command_id[MAX_INPUT_BUFFER];
	char args[MAX_INPUT_BUFFER];
	time_t entry_time;
	int command_type=CMD_NONE;
	char *temp_ptr;

#ifdef DEBUG0
	printf("check_for_external_commands() start\n");
#endif

	/* bail out if we shouldn't be checking for external commands */
	if(!check_external_commands)
		return;

	/* update last command check time */
	last_command_check=time(NULL);

	/* update the status log with new program information */
	update_status_log_with_program();

	/* try and open the command file for reading */
	fp=fopen(command_file,"r");
	if(fp==NULL){
#ifdef DEBUG3
		printf("\tExternal command file not found.\n");
#endif
		return;
	        }

#ifdef DEBUG3
		printf("\tCommand file present - checking command entries...\n");
#endif

	/* process all commands in the file */
	for(fgets(buffer,(int)(sizeof(buffer)-1),fp);!feof(fp);fgets(buffer,(int)(sizeof(buffer)-1),fp)){

		/* strip the buffer of newlines and carriage returns */
		strip(buffer);

#ifdef DEBUG3
		printf("\tRaw command entry: %s\n",buffer);
#endif

		/* get the command entry time */
		temp_ptr=my_strtok(buffer,"[");
		if(temp_ptr==NULL)
			continue;
		temp_ptr=my_strtok(NULL,"]");
		if(temp_ptr==NULL)
			continue;
		entry_time=(time_t)strtoul(temp_ptr,NULL,10);

		/* get the command identifier */
		temp_ptr=my_strtok(NULL,";");
		if(temp_ptr==NULL)
			continue;
		strcpy(command_id,temp_ptr+1);

		/* get the command arguments */
		temp_ptr=my_strtok(NULL,"\n");
		if(temp_ptr==NULL)
			strcpy(args,"");
		else{
			strncpy(args,temp_ptr,sizeof(args));
			args[sizeof(args)-1]='\x0';
		        }

		/* decide what type of command this is */
		if(!strcmp(command_id,"ADD_HOST_COMMENT"))
			command_type=CMD_ADD_HOST_COMMENT;
		else if(!strcmp(command_id,"ADD_SVC_COMMENT"))
			command_type=CMD_ADD_SVC_COMMENT;

		else if(!strcmp(command_id,"DEL_HOST_COMMENT"))
			command_type=CMD_DEL_HOST_COMMENT;
		else if(!strcmp(command_id,"DEL_SVC_COMMENT"))
			command_type=CMD_DEL_SVC_COMMENT;

		else if(!strcmp(command_id,"DELAY_HOST_NOTIFICATION"))
			command_type=CMD_DELAY_HOST_NOTIFICATION;
		else if(!strcmp(command_id,"DELAY_SVC_NOTIFICATION"))
			command_type=CMD_DELAY_SVC_NOTIFICATION;

		/* this is used for immediate and delayed service checks... */
		else if(!strcmp(command_id,"SCHEDULE_SVC_CHECK"))
			command_type=CMD_DELAY_SVC_CHECK;

		else if(!strcmp(command_id,"ENABLE_SVC_CHECK"))
			command_type=CMD_ENABLE_SVC_CHECK;
		else if(!strcmp(command_id,"DISABLE_SVC_CHECK"))
			command_type=CMD_DISABLE_SVC_CHECK;

		else if(!strcmp(command_id,"ENTER_STANDBY_MODE"))
			command_type=CMD_ENTER_STANDBY_MODE;
		else if(!strcmp(command_id,"ENTER_ACTIVE_MODE"))
			command_type=CMD_ENTER_ACTIVE_MODE;

		else if(!strcmp(command_id,"SHUTDOWN_PROGRAM"))
			command_type=CMD_SHUTDOWN_PROCESS;
		else if(!strcmp(command_id,"RESTART_PROGRAM"))
			command_type=CMD_RESTART_PROCESS;

		else if(!strcmp(command_id,"ENABLE_HOST_SVC_CHECKS"))
			command_type=CMD_ENABLE_HOST_SVC_CHECKS;
		else if(!strcmp(command_id,"DISABLE_HOST_SVC_CHECKS"))
			command_type=CMD_DISABLE_HOST_SVC_CHECKS;

		/* this is used for immediate and delayed service checks... */
		else if(!strcmp(command_id,"SCHEDULE_HOST_SVC_CHECKS"))
			command_type=CMD_DELAY_HOST_SVC_CHECKS;

		else if(!strcmp(command_id,"DEL_ALL_HOST_COMMENTS"))
			command_type=CMD_DEL_ALL_HOST_COMMENTS;
		else if(!strcmp(command_id,"DEL_ALL_SVC_COMMENTS"))
			command_type=CMD_DEL_ALL_SVC_COMMENTS;

		else if(!strcmp(command_id,"ENABLE_SVC_NOTIFICATIONS"))
			command_type=CMD_ENABLE_SVC_NOTIFICATIONS;
		else if(!strcmp(command_id,"DISABLE_SVC_NOTIFICATIONS"))
			command_type=CMD_DISABLE_SVC_NOTIFICATIONS;

		else if(!strcmp(command_id,"ENABLE_HOST_NOTIFICATIONS"))
			command_type=CMD_ENABLE_HOST_NOTIFICATIONS;
		else if(!strcmp(command_id,"DISABLE_HOST_NOTIFICATIONS"))
			command_type=CMD_DISABLE_HOST_NOTIFICATIONS;

		else if(!strcmp(command_id,"ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST"))
			command_type=CMD_ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST;
		else if(!strcmp(command_id,"DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST"))
			command_type=CMD_DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST;

		else if(!strcmp(command_id,"ENABLE_HOST_SVC_NOTIFICATIONS"))
			command_type=CMD_ENABLE_HOST_SVC_NOTIFICATIONS;
		else if(!strcmp(command_id,"DISABLE_HOST_SVC_NOTIFICATIONS"))
			command_type=CMD_DISABLE_HOST_SVC_NOTIFICATIONS;

		else
			continue;

		/* log the external command */
		snprintf(buffer,sizeof(buffer),"EXTERNAL COMMAND: %s;%s\n",command_id,args);
		buffer[sizeof(buffer)-1]='\x0';
		write_to_log(buffer);

		/* process the command */
		process_external_command(command_type,entry_time,args);
	        }

	fclose(fp);

	/* delete the command file */
	remove(command_file);

#ifdef DEBUG0
	printf("check_for_external_commands() end\n");
#endif

	return;
        }



/* top-level processor for a single external command */
void process_external_command(int cmd,time_t entry_time,char *args){

#ifdef DEBUG0
	printf("process_external_command() start\n");
#endif

#ifdef DEBUG3
	printf("\tExternal Command Type: %d\n",cmd);
	printf("\tCommand Entry Time: %lu\n",(unsigned long)entry_time);
	printf("\tCommand Arguments: %s\n",args);
#endif

	/* how shall we execute the command? */
	switch(cmd){

	case CMD_ADD_HOST_COMMENT:
	case CMD_ADD_SVC_COMMENT:
		cmd_add_comment(cmd,entry_time,args);
		break;

	case CMD_DEL_HOST_COMMENT:
	case CMD_DEL_SVC_COMMENT:
		cmd_delete_comment(cmd,args);
		break;

	case CMD_DELAY_HOST_NOTIFICATION:
	case CMD_DELAY_SVC_NOTIFICATION:
		cmd_delay_notification(cmd,args);
		break;

	case CMD_DELAY_SVC_CHECK:
	case CMD_IMMEDIATE_SVC_CHECK:
		cmd_schedule_service_check(cmd,args);
		break;

	case CMD_ENABLE_SVC_CHECK:
		cmd_enable_service_check(cmd,args);
		break;

	case CMD_DISABLE_SVC_CHECK:
		cmd_disable_service_check(cmd,args);
		break;

	case CMD_ENTER_STANDBY_MODE:
	case CMD_ENTER_ACTIVE_MODE:
		cmd_set_program_mode(cmd,args);
		break;

	case CMD_SHUTDOWN_PROCESS:
	case CMD_RESTART_PROCESS:
		cmd_signal_process(cmd,args);
		break;

	case CMD_ENABLE_HOST_SVC_CHECKS:
		cmd_enable_host_service_checks(cmd,args);
		break;

	case CMD_DISABLE_HOST_SVC_CHECKS:
		cmd_disable_host_service_checks(cmd,args);
		break;

	case CMD_DELAY_HOST_SVC_CHECKS:
	case CMD_IMMEDIATE_HOST_SVC_CHECKS:
		cmd_schedule_host_service_checks(cmd,args);
		break;

	case CMD_DEL_ALL_HOST_COMMENTS:
	case CMD_DEL_ALL_SVC_COMMENTS:
		cmd_delete_all_comments(cmd,args);
		break;

	case CMD_ENABLE_SVC_NOTIFICATIONS:
		cmd_enable_service_notifications(cmd,args);
		break;

	case CMD_DISABLE_SVC_NOTIFICATIONS:
		cmd_disable_service_notifications(cmd,args);
		break;

	case CMD_ENABLE_HOST_NOTIFICATIONS:
		cmd_enable_host_notifications(cmd,args);
		break;

	case CMD_DISABLE_HOST_NOTIFICATIONS:
		cmd_disable_host_notifications(cmd,args);
		break;

	case CMD_ENABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		cmd_enable_and_propagate_notifications(cmd,args);
		break;

	case CMD_DISABLE_ALL_NOTIFICATIONS_BEYOND_HOST:
		cmd_disable_and_propagate_notifications(cmd,args);
		break;

	case CMD_ENABLE_HOST_SVC_NOTIFICATIONS:
		cmd_enable_host_service_notifications(cmd,args);
		break;

	case CMD_DISABLE_HOST_SVC_NOTIFICATIONS:
		cmd_disable_host_service_notifications(cmd,args);
		break;

	default:
		break;
	        }

#ifdef DEBUG0
	printf("process_external_command() end\n");
#endif

	return;
        }



/******************************************************************/
/*************** EXTERNAL COMMAND IMPLEMENTATIONS  ****************/
/******************************************************************/

/* adds a host or service comment to the status log */
int cmd_add_comment(int cmd,time_t entry_time,char *args){
	char *temp_ptr;
	host *temp_host;
	service *temp_service;
	FILE *fp;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];
	char user[MAX_INPUT_BUFFER];
	char comment[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	int persistent=0;

#ifdef DEBUG0
	printf("cmd_add_comment() start\n");
#endif

	/* get the host nmae */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* if we're adding a service comment...  */
	if(cmd==CMD_ADD_SVC_COMMENT){

		/* get the service description */
		temp_ptr=my_strtok(NULL,";");
		if(temp_ptr==NULL)
			return ERROR;
		strncpy(service,temp_ptr,sizeof(service));
		service[sizeof(service)-1]='\x0';

		/* verify that the service is valid */
		temp_service=find_service(host,service,NULL);
		if(temp_service==NULL)
			return ERROR;
	        }

	/* else verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* get the persistant flag */
	temp_ptr=my_strtok(NULL,";");
	if(temp_ptr==NULL)
		return ERROR;
	persistent=atoi(temp_ptr);

	/* get the name of the user who entered the comment */
	temp_ptr=my_strtok(NULL,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(user,temp_ptr,sizeof(user));
	user[sizeof(user)-1]='\x0';

	/* get the comment */
	temp_ptr=my_strtok(NULL,"\n");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(comment,temp_ptr,sizeof(comment));
	comment[sizeof(comment)-1]='\x0';

	/* increment the current comment id */
	current_comment_id++;

	/* create the comment string */
	if(cmd==CMD_ADD_HOST_COMMENT)
		snprintf(temp_buffer,sizeof(temp_buffer)-1,"[%lu] HOST_COMMENT;%d;%s;%d;%s;%s\n",(unsigned long)entry_time,current_comment_id,host,persistent,user,comment);
	else
		snprintf(temp_buffer,sizeof(temp_buffer)-1,"[%lu] SERVICE_COMMENT;%d;%s;%s;%d;%s;%s\n",(unsigned long)entry_time,current_comment_id,host,service,persistent,user,comment);
	temp_buffer[sizeof(temp_buffer)-1]='\x0';
	
	/* open the status log to add the comment */
	fp=fopen(comment_file,"a");
	if(fp==NULL)
		return ERROR;
	fputs(temp_buffer,fp);
	fclose(fp);

#ifdef DEBUG0
	printf("cmd_add_comment() end\n");
#endif
	return OK;
        }



/* removes a host or service comment from the status log */
int cmd_delete_comment(int cmd,char *args){
	FILE *fpin;
	FILE *fpout;
	char buffer[MAX_INPUT_BUFFER];
	char match[MAX_INPUT_BUFFER];
	int comment_id;

#ifdef DEBUG0
	printf("cmd_del_comment() start\n");
#endif
	
	/* get the comment id we should delete */
	comment_id=atoi(args);

	/* create the matching string we're looking for in the log file */
	if(cmd==CMD_DEL_HOST_COMMENT)
		snprintf(match,sizeof(match),"] HOST_COMMENT;%d;",comment_id);
	else
		snprintf(match,sizeof(match),"] SERVICE_COMMENT;%d;",comment_id);
	match[sizeof(match)-1]='\x0';

	fpin=fopen(comment_file,"r");
	if(fpin==NULL){
#ifdef DEBUG1
		printf("Error: Could not open comment log for reading!\n");
#endif
		return ERROR;
	        }

	fpout=fopen(temp_file,"w");
	if(fpout==NULL){
#ifdef DEBUG1
		printf("Error: Could not open temp file for writing!\n");
#endif
		fclose(fpin);
		return ERROR;
	        }

	/* write every line in the comment file, except the matching comment */
	while(fgets(buffer,(int)(sizeof(buffer)-1),fpin)){

		if(feof(fpin))
			break;

		if(!strstr(buffer,match))
			fputs(buffer,fpout);
	        }

	fclose(fpout);
	fclose(fpin);

	rename(temp_file,comment_file);

#ifdef DEBUG0
	printf("cmd_del_comment() end\n");
#endif
	return OK;
        }



/* removes all comments associated with a host or service from the status log */
int cmd_delete_all_comments(int cmd,char *args){
	FILE *fpin;
	FILE *fpout;
	char *temp_ptr;
	service *temp_service=NULL;
	host *temp_host=NULL;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];
	char buffer[MAX_INPUT_BUFFER];
	char match1[MAX_INPUT_BUFFER];
	char match2[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_del_all_comments() start\n");
#endif
	
	/* get the host nmae */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* if we're deleting service comments...  */
	if(cmd==CMD_DEL_ALL_SVC_COMMENTS){

		/* get the service description */
		temp_ptr=my_strtok(NULL,";");
		if(temp_ptr==NULL)
			return ERROR;
		strncpy(service,temp_ptr,sizeof(service));
		service[sizeof(service)-1]='\x0';

		/* verify that the service is valid */
		temp_service=find_service(host,service,NULL);
		if(temp_service==NULL)
			return ERROR;
	        }

	/* else verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;


	/* create the matching string we're looking for in the log file */
	if(cmd==CMD_DEL_ALL_HOST_COMMENTS){
		snprintf(match1,sizeof(match1),"] HOST_COMMENT;");
		snprintf(match2,sizeof(match2),";%s;",host);
	        }
	else{
		snprintf(match1,sizeof(match1),"] SERVICE_COMMENT;");
		snprintf(match2,sizeof(match2),";%s;%s;",host,service);
	        }
	match1[sizeof(match1)-1]='\x0';
	match2[sizeof(match2)-1]='\x0';

	fpin=fopen(comment_file,"r");
	if(fpin==NULL){
#ifdef DEBUG1
		printf("Error: Could not open comment log for reading!\n");
#endif
		return ERROR;
	        }

	fpout=fopen(temp_file,"w");
	if(fpout==NULL){
#ifdef DEBUG1
		printf("Error: Could not open temp file for writing!\n");
#endif
		fclose(fpin);
		return ERROR;
	        }

	/* write every line in the comment file, except the matching comment(s) */
	while(fgets(buffer,(int)(sizeof(buffer)-1),fpin)){

		if(feof(fpin))
			break;

		if(!(strstr(buffer,match1) && strstr(buffer,match2)))
			fputs(buffer,fpout);
	        }

	fclose(fpout);
	fclose(fpin);

	rename(temp_file,comment_file);

#ifdef DEBUG0
	printf("cmd_del_all_comments() end\n");
#endif
	return OK;
        }



/* delays a host or service notification for given number of minutes */
int cmd_delay_notification(int cmd,char *args){
	char *temp_ptr;
	host *temp_host=NULL;
	service *temp_service=NULL;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];
	time_t delay_time;

#ifdef DEBUG0
	printf("cmd_delay_notification() start\n");
#endif

	/* get the host nmae */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* if this is a service notification delay...  */
	if(cmd==CMD_DELAY_SVC_NOTIFICATION){

		/* get the service description */
		temp_ptr=my_strtok(NULL,";");
		if(temp_ptr==NULL)
			return ERROR;
		strncpy(service,temp_ptr,sizeof(service));
		service[sizeof(service)-1]='\x0';

		/* verify that the service is valid */
		temp_service=find_service(host,service,NULL);
		if(temp_service==NULL)
			return ERROR;
	        }

	/* else verify that the host is valid */
	else{

		temp_host=find_host(host,NULL);
		if(temp_host==NULL)
			return ERROR;
	        }

	/* get the time that we should delay until... */
	temp_ptr=my_strtok(NULL,"\n");
	if(temp_ptr==NULL)
		return ERROR;
	delay_time=strtoul(temp_ptr,NULL,10);

	/* delay the next notification... */
	if(cmd==CMD_DELAY_HOST_NOTIFICATION)
		temp_host->last_host_notification=delay_time;
	else
		temp_service->last_notification=delay_time;
	

#ifdef DEBUG0
	printf("cmd_delay_notification() end\n");
#endif
	return OK;
        }



/* schedules a service check at a particular time */
int cmd_schedule_service_check(int cmd,char *args){
	char *temp_ptr;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];
	time_t delay_time=0L;

#ifdef DEBUG0
	printf("cmd_schedule_service_check() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* get the service description */
	temp_ptr=my_strtok(NULL,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(service,temp_ptr,sizeof(service));
	service[sizeof(service)-1]='\x0';

	/* verify that the service is valid */
	temp_service=find_service(host,service,NULL);
	if(temp_service==NULL)
		return ERROR;

	/* get the next check time */
	temp_ptr=my_strtok(NULL,"\n");
	if(temp_ptr==NULL)
		return ERROR;
	delay_time=strtoul(temp_ptr,NULL,10);

	/* schedule a delayed service check */
	schedule_service_check(temp_service,delay_time);

#ifdef DEBUG0
	printf("cmd_schedule_service_check() end\n");
#endif
	return OK;
        }



/* schedules all service checks on a host for a particular time */
int cmd_schedule_host_service_checks(int cmd,char *args){
	char *temp_ptr;
	host *temp_host;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];
	time_t delay_time=0L;

#ifdef DEBUG0
	printf("cmd_schedule_host_service_checks() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* get the next check time */
	temp_ptr=my_strtok(NULL,"\n");
	if(temp_ptr==NULL)
		return ERROR;
	delay_time=strtoul(temp_ptr,NULL,10);

	/* reschedule all services on the specified host */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		/* schedule a delayed service check */
		if(!strcmp(host,temp_service->host_name))
			schedule_service_check(temp_service,delay_time);
	        }


#ifdef DEBUG0
	printf("cmd_schedule_host_service_checks() end\n");
#endif
	return OK;
        }



/* enables a previously disabled service check */
int cmd_enable_service_check(int cmd,char *args){
	char *temp_ptr;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_enable_service_check() start\n");
#endif

	/* get the host nmae */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* get the service description */
	temp_ptr=my_strtok(NULL,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(service,temp_ptr,sizeof(service));
	service[sizeof(service)-1]='\x0';

	/* verify that the service is valid */
	temp_service=find_service(host,service,NULL);
	if(temp_service==NULL)
		return ERROR;

	/* enable the service check */
	enable_service_check(temp_service);

#ifdef DEBUG0
	printf("cmd_enable_service_check() end\n");
#endif
	return OK;
        }



/* temporarily disables a service check */
int cmd_disable_service_check(int cmd,char *args){
	char *temp_ptr;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_disable_service_check() start\n");
#endif

	/* get the host nmae */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* get the service description */
	temp_ptr=my_strtok(NULL,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(service,temp_ptr,sizeof(service));
	service[sizeof(service)-1]='\x0';

	/* verify that the service is valid */
	temp_service=find_service(host,service,NULL);
	if(temp_service==NULL)
		return ERROR;

	disable_service_check(temp_service);

#ifdef DEBUG0
	printf("cmd_disable_service_check() end\n");
#endif
	return OK;
        }



/* enables all service checks for a host */
int cmd_enable_host_service_checks(int cmd,char *args){
	char *temp_ptr;
	host *temp_host;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_enable_host_service_checks() start\n");
#endif

	/* get the host nmae */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* disable all services associated with this host... */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		if(!strcmp(temp_service->host_name,temp_host->name))
			enable_service_check(temp_service);
	        }

#ifdef DEBUG0
	printf("cmd_enable_host_service_checks() end\n");
#endif
	return OK;
        }



/* temporarily disables all service checks for a host */
int cmd_disable_host_service_checks(int cmd,char *args){
	char *temp_ptr;
	host *temp_host;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_disable_host_service_checks() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* disable all services associated with this host... */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		if(!strcmp(temp_service->host_name,temp_host->name))
			disable_service_check(temp_service);
	        }

#ifdef DEBUG0
	printf("cmd_disable_host_service_checks() end\n");
#endif
	return OK;
        }



/* schedules a program shutdown or restart */
int cmd_signal_process(int cmd, char *args){
	timed_event *new_event;
	time_t scheduled_time;
	char *temp_ptr;

#ifdef DEBUG0
	printf("cmd_signal_process() start\n");
#endif

	/* get the time to schedule the event */
	temp_ptr=my_strtok(args,"\n");
	if(temp_ptr==NULL)
		scheduled_time=0L;
	else
		scheduled_time=strtoul(temp_ptr,NULL,10);

	/* add a scheduled program shutdown or restart to the event list */
	new_event=malloc(sizeof(timed_event));
	if(new_event!=NULL){
		new_event->event_type=(cmd==CMD_SHUTDOWN_PROCESS)?EVENT_PROGRAM_SHUTDOWN:EVENT_PROGRAM_RESTART;
		new_event->event_data=(void *)NULL;
		new_event->run_time=scheduled_time;
		new_event->recurring=FALSE;
		schedule_event(new_event,&event_list_high);
	        }
	else
		return ERROR;

#ifdef DEBUG0
	printf("cmd_signal_process() end\n");
#endif

	return OK;
        }


int cmd_set_program_mode(int cmd, char *args){
	timed_event *new_event;
	time_t scheduled_time;
	char *temp_ptr;

#ifdef DEBUG0
	printf("cmd_set_program_mode() start\n");
#endif

	/* get the time to schedule the event */
	temp_ptr=my_strtok(args,"\n");
	if(temp_ptr==NULL)
		scheduled_time=0L;
	else
		scheduled_time=strtoul(temp_ptr,NULL,10);

	/* add a scheduled program mode change */
	new_event=malloc(sizeof(timed_event));
	if(new_event!=NULL){
		new_event->event_type=(cmd==CMD_ENTER_ACTIVE_MODE)?EVENT_ENTER_ACTIVE_MODE:EVENT_ENTER_STANDBY_MODE;
		new_event->event_data=(void *)NULL;
		new_event->run_time=scheduled_time;
		new_event->recurring=FALSE;
		schedule_event(new_event,&event_list_high);
	        }
	else
		return ERROR;

#ifdef DEBUG0
	printf("cmd_set_program_mode() end\n");
#endif

	return OK;
        }


/* enables notifications for a service */
int cmd_enable_service_notifications(int cmd,char *args){
	char *temp_ptr;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_enable_service_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* get the service description */
	temp_ptr=my_strtok(NULL,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(service,temp_ptr,sizeof(service));
	service[sizeof(service)-1]='\x0';

	/* verify that the service is valid */
	temp_service=find_service(host,service,NULL);
	if(temp_service==NULL)
		return ERROR;

	/* enable notifications for this service */
	enable_service_notifications(temp_service);

#ifdef DEBUG0
	printf("cmd_enable_service_notifications() end\n");
#endif
	return OK;
        }


/* temporarily disables notifications for a service */
int cmd_disable_service_notifications(int cmd,char *args){
	char *temp_ptr;
	service *temp_service;
	char host[MAX_INPUT_BUFFER];
	char service[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_disable_service_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* get the service description */
	temp_ptr=my_strtok(NULL,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(service,temp_ptr,sizeof(service));
	service[sizeof(service)-1]='\x0';

	/* verify that the service is valid */
	temp_service=find_service(host,service,NULL);
	if(temp_service==NULL)
		return ERROR;

	/* disable notifications for this service */
	disable_service_notifications(temp_service);

#ifdef DEBUG0
	printf("cmd_disable_service_notifications() end\n");
#endif
	return OK;
        }


/* enables notifications for a host */
int cmd_enable_host_notifications(int cmd,char *args){
	char *temp_ptr;
	host *temp_host;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_enable_host_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* enable notifications for this host */
	enable_host_notifications(temp_host);

#ifdef DEBUG0
	printf("cmd_enable_host_notifications() end\n");
#endif
	return OK;
        }


/* temporarily disables notifications for a host */
int cmd_disable_host_notifications(int cmd,char *args){
	char *temp_ptr;
	host *temp_host;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_disable_host_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* disable notifications for this host */
	disable_host_notifications(temp_host);

#ifdef DEBUG0
	printf("cmd_disable_host_notifications() end\n");
#endif
	return OK;
        }


/* enables notifications for all services on a host  */
int cmd_enable_host_service_notifications(int cmd,char *args){
	char *temp_ptr;
	service *temp_service;
	host *temp_host;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_enable_host_service_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* find all services for this host... */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		if(!strcmp(temp_service->host_name,host))
			enable_service_notifications(temp_service);
	        }

#ifdef DEBUG0
	printf("cmd_enable_host_service_notifications() end\n");
#endif
	return OK;
        }


/* temporarily disables notifications for all services on a host  */
int cmd_disable_host_service_notifications(int cmd,char *args){
	char *temp_ptr;
	service *temp_service;
	host *temp_host;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_disable_host_service_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* find all services for this host... */
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		if(!strcmp(temp_service->host_name,host))
			disable_service_notifications(temp_service);
	        }

#ifdef DEBUG0
	printf("cmd_disable_host_service_notifications() end\n");
#endif
	return OK;
        }


/* enables notifications for all hosts and services "beyond" a given host */
int cmd_enable_and_propagate_notifications(int cmd,char *args){
	char *temp_ptr;
	host *temp_host;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_enable_and_propagate_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* enable and propagate notifications.. */
	enable_and_propagate_notifications(temp_host);

#ifdef DEBUG0
	printf("cmd_enable_propagate_notifications() end\n");
#endif
	return OK;
        }


/* disables notifications for all hosts and services "beyond" a given host */
int cmd_disable_and_propagate_notifications(int cmd,char *args){
	char *temp_ptr;
	host *temp_host;
	char host[MAX_INPUT_BUFFER];

#ifdef DEBUG0
	printf("cmd_disable_and_propagate_notifications() start\n");
#endif

	/* get the host name */
	temp_ptr=my_strtok(args,";");
	if(temp_ptr==NULL)
		return ERROR;
	strncpy(host,temp_ptr,sizeof(host));
	host[sizeof(host)-1]='\x0';

	/* verify that the host is valid */
	temp_host=find_host(host,NULL);
	if(temp_host==NULL)
		return ERROR;

	/* disable and propagate notifications.. */
	disable_and_propagate_notifications(temp_host);

#ifdef DEBUG0
	printf("cmd_disable_propagate_notifications() end\n");
#endif
	return OK;
        }





/******************************************************************/
/*************** INTERNAL COMMAND IMPLEMENTATIONS  ****************/
/******************************************************************/

/* temporarily disables a service check */
void disable_service_check(service *svc){

#ifdef DEBUG0
	printf("disable_service_check() start\n");
#endif
	/* disable the service check... */
	svc->disabled=TRUE;

	/* update the status log to reflect the new service state */
	update_status_log_with_service(svc);

#ifdef DEBUG0
	printf("disable_service_check() end\n");
#endif

	return;
        }



/* enables a service check */
void enable_service_check(service *svc){

#ifdef DEBUG0
	printf("enable_service_check() start\n");
#endif

	/* enable the service check... */
	svc->disabled=FALSE;

	/* update the status log to reflect the new service state */
	update_status_log_with_service(svc);

#ifdef DEBUG0
	printf("enable_service_check() end\n");
#endif

	return;
        }



/* schedules an immediate or delayed service check */
void schedule_service_check(service *svc,time_t check_time){
	timed_event *temp_event;

#ifdef DEBUG0
	printf("schedule_service_check() start\n");
#endif

	/* find the service check event in the queue... */
	for(temp_event=event_list_low;temp_event!=NULL;temp_event=temp_event->next){

		if(temp_event->event_type==EVENT_SERVICE_CHECK && svc==(service *)temp_event->event_data)
			break;
	        }

	/* bail out if we couldn't find the original service check event */
	if(temp_event==NULL)
		return;

	/* set the next service check time */
	svc->next_check=check_time;

	/* remove the original service check from the event queue */
	remove_event(temp_event,&event_list_low);

	/* reschedule the next service check */
	schedule_event(temp_event,&event_list_low);

	/* update the status log */
	update_status_log_with_service(svc);

#ifdef DEBUG0
	printf("schedule_service_check() end\n");
#endif

	return;
        }


/* enables notifications for a service */
void enable_service_notifications(service *svc){

#ifdef DEBUG0
	printf("enable_service_notifications() start\n");
#endif

	/* enable the service notifications... */
	svc->notifications_disabled=FALSE;

	/* update the status log to reflect the new service state */
	update_status_log_with_service(svc);

#ifdef DEBUG0
	printf("enable_service_notifications() end\n");
#endif

	return;
        }


/* disables notifications for a service */
void disable_service_notifications(service *svc){

#ifdef DEBUG0
	printf("disable_service_notifications() start\n");
#endif

	/* enable the service notifications... */
	svc->notifications_disabled=TRUE;

	/* update the status log to reflect the new service state */
	update_status_log_with_service(svc);

#ifdef DEBUG0
	printf("disable_service_notifications() end\n");
#endif

	return;
        }


/* enables notifications for a host */
void enable_host_notifications(host *hst){

#ifdef DEBUG0
	printf("enable_host_notifications() start\n");
#endif

	/* enable the host notifications... */
	hst->notifications_disabled=FALSE;

	/* update the status log to reflect the new host state */
	update_status_log_with_host(hst);

#ifdef DEBUG0
	printf("enable_host_notifications() end\n");
#endif

	return;
        }


/* disables notifications for a host */
void disable_host_notifications(host *hst){

#ifdef DEBUG0
	printf("disable_host_notifications() start\n");
#endif

	/* disable the host notifications... */
	hst->notifications_disabled=TRUE;

	/* update the status log to reflect the new host state */
	update_status_log_with_host(hst);

#ifdef DEBUG0
	printf("disable_host_notifications() end\n");
#endif

	return;
        }


/* enables notifications for all hosts and services "beyond" a given host */
void enable_and_propagate_notifications(host *hst){
	host *temp_host;
	service *temp_service;

#ifdef DEBUG0
	printf("enable_and_propagate_notifications() start\n");
#endif

	/* check all child hosts... */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		if(is_immediate_child_of_host(hst,temp_host)==TRUE){

			/* recurse... */
			enable_and_propagate_notifications(temp_host);

			/* enable notifications for this host */
			enable_host_notifications(temp_host);

			/* enable notifications for all services on this host... */
			for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
				if(!strcmp(temp_host->name,temp_service->host_name))
					enable_service_notifications(temp_service);
			        }
		        }

	        }

#ifdef DEBUG0
	printf("enable_and_propagate_notifications() end\n");
#endif

	return;
        }


/* disables notifications for all hosts and services "beyond" a given host */
void disable_and_propagate_notifications(host *hst){
	host *temp_host;
	service *temp_service;

#ifdef DEBUG0
	printf("disable_and_propagate_notifications() start\n");
#endif

	/* check all child hosts... */
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		if(is_immediate_child_of_host(hst,temp_host)==TRUE){

			/* recurse... */
			disable_and_propagate_notifications(temp_host);

			/* disable notifications for this host */
			disable_host_notifications(temp_host);

			/* disable notifications for all services on this host... */
			for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
				if(!strcmp(temp_host->name,temp_service->host_name))
					disable_service_notifications(temp_service);
			        }
		        }

	        }

#ifdef DEBUG0
	printf("disable_and_propagate_notifications() end\n");
#endif

	return;
        }



