/*****************************************************************************
 *
 * CONFIG.C - Configuration input and verification routines for NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   01-14-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netsaint.h"


extern char	config_file[MAX_FILENAME_LENGTH];
extern char     hosts_config_file[MAX_FILENAME_LENGTH];
extern char	log_file[MAX_FILENAME_LENGTH];
extern char     status_file[MAX_FILENAME_LENGTH];
extern char     command_file[MAX_FILENAME_LENGTH];
extern char     temp_file[MAX_FILENAME_LENGTH];
extern char     comment_file[MAX_FILENAME_LENGTH];
extern char     log_archive_path[MAX_FILENAME_LENGTH];

extern char     generic_summary[MAX_COMMANDLINE_LENGTH];

extern char     macro_admin_email[MAX_EMAIL_LENGTH];
extern char     macro_admin_pager[MAX_PAGER_LENGTH];

extern char     global_host_event_handler[MAX_COMMANDNAME_LENGTH];
extern char     global_service_event_handler[MAX_COMMANDNAME_LENGTH];

extern int	log_level;
extern int	use_syslog;
extern int	syslog_level;
extern int      log_notifications;
extern int      log_service_retries;
extern int      log_host_retries;
extern int      log_event_handlers;

extern int      sleep_time;
extern int      interval_length;
extern int      inter_check_delay_method;
extern int      interleave_factor;
extern int      interleave_factor_method;

extern int      max_child_process_time;

extern int      max_parallel_service_checks;

extern int      zombie_cleanup_interval;
extern int      command_check_interval;
extern int      service_check_reaper_interval;

extern int      check_external_commands;

extern int      use_agressive_host_checking;

extern int      log_rotation_method;

extern int      program_mode;

extern contact		*contact_list;
extern contactgroup	*contactgroup_list;
extern host		*host_list;
extern hostgroup	*hostgroup_list;
extern service		*service_list;
extern notification     *notification_list;
extern command          *command_list;
extern timeperiod       *timeperiod_list;





/******************************************************************/
/************** CONFIGURATION INPUT FUNCTIONS *********************/
/******************************************************************/

/* process the main configuration file */
int read_main_config(char *config_file){
	char input[MAX_INPUT_BUFFER];
	char variable[MAX_INPUT_BUFFER];
	char value[MAX_INPUT_BUFFER];
	char *temp;
	FILE *fp;
	int current_line=0;
	int result;

#ifdef DEBUG0
	printf("read_main_config() start\n");
#endif
#ifdef DEBUG1
	printf("\tConfig file: %s\n",config_file);
#endif

	/* open the config file for reading */
	fp=fopen(config_file,"r");
	if(fp==NULL){
		printf("Error: Cannot open main configuration file '%s' for reading!\n",config_file);
		return ERROR;
		}

	/* process all lines in the config file */
	for(current_line=1,fgets(input,MAX_INPUT_BUFFER-1,fp);!feof(fp);current_line++,fgets(input,MAX_INPUT_BUFFER-1,fp)){

		/* skip blank lines and comments */
		if(input[0]=='#' || input[0]=='\x0' || input[0]=='\n' || input[0]=='\r')
			continue;

#ifdef DEBUG1
		printf("\tLine %d Raw Input: %s",current_line,input);
#endif

		strip(input);

#ifdef DEBUG1
		printf("\tLine %d Stripped Input: %s\n",current_line,input);
#endif

		/* get the variable name */
		temp=my_strtok(input,"=");

		/* if there is no variable name, return error */
		if(temp==NULL){

			printf("Error in main configuration file '%s' - Line %d (NULL variable)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		/* else the variable is good */
		strncpy(variable,temp,sizeof(variable));
		variable[sizeof(variable)-1]='\x0';

		/* get the value */
		temp=my_strtok(NULL,"=");

		/* if no value exists, return error */
		if(temp==NULL){

			printf("Error in main configuration file '%s' - Line %d (NULL value)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		/* else the value is good */
		strncpy(value,temp,sizeof(value));
		value[sizeof(value)-1]='\x0';
		strip(value);

		/* process the variable/value */

		if(!strcmp(variable,"cfg_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Config filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(hosts_config_file,value,sizeof(hosts_config_file));
			hosts_config_file[sizeof(hosts_config_file)-1]='\x0';
			strip(hosts_config_file);

#ifdef DEBUG1
			printf("\t\thosts_config_file set to '%s'\n",hosts_config_file);
#endif

			result=read_config_file(hosts_config_file);
			if(result!=OK){
				printf("An error occurred while processing host config file '%s'\n",hosts_config_file);
				fclose(fp);
				return result;
				}
			}
		else if(!strcmp(variable,"log_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Log filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(log_file,value,sizeof(log_file));
			log_file[sizeof(log_file)-1]='\x0';
			strip(log_file);

#ifdef DEBUG1
			printf("\t\tlog_file set to '%s'\n",log_file);
#endif
			}
		else if(!strcmp(variable,"command_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Command filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(command_file,value,sizeof(command_file));
			command_file[sizeof(command_file)-1]='\x0';
			strip(command_file);

#ifdef DEBUG1
			printf("\t\tcommand_file set to '%s'\n",command_file);
#endif
			}
		else if(!strcmp(variable,"temp_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Temp filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(temp_file,value,sizeof(temp_file));
			temp_file[sizeof(temp_file)-1]='\x0';
			strip(temp_file);

#ifdef DEBUG1
			printf("\t\ttemp_file set to '%s'\n",temp_file);
#endif
			}
		else if(!strcmp(variable,"status_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Status filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(status_file,value,sizeof(status_file));
			status_file[sizeof(status_file)-1]='\x0';
			strip(status_file);

#ifdef DEBUG1
			printf("\t\tstatus_file set to '%s'\n",status_file);
#endif
			}
		else if(!strcmp(variable,"comment_file")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Comment filename too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(comment_file,value,sizeof(comment_file));
			comment_file[sizeof(comment_file)-1]='\x0';
			strip(comment_file);

#ifdef DEBUG1
			printf("\t\tcomment_file set to '%s'\n",comment_file);
#endif
			}
		else if(!strcmp(variable,"generic_summary")){
			if(strlen(value)>MAX_COMMANDLINE_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Generic summary too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(generic_summary,value,sizeof(generic_summary));
			generic_summary[sizeof(generic_summary)-1]='\x0';
			strip(generic_summary);

#ifdef DEBUG1
			printf("\t\tgeneric_summary set to '%s'\n",generic_summary);
#endif
		        }
		else if(!strcmp(variable,"global_host_event_handler")){
			if(strlen(value)>MAX_COMMANDNAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Global host event handler command too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(global_host_event_handler,value,sizeof(global_host_event_handler));
			global_host_event_handler[sizeof(global_host_event_handler)-1]='\x0';
			strip(global_host_event_handler);

#ifdef DEBUG1
			printf("\t\tglobal_host_event_handler set to '%s'\n",global_host_event_handler);
#endif
		        }
		else if(!strcmp(variable,"global_service_event_handler")){
			if(strlen(value)>MAX_COMMANDNAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Global service event handler command too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(global_service_event_handler,value,sizeof(global_service_event_handler));
			global_service_event_handler[sizeof(global_service_event_handler)-1]='\x0';
			strip(global_service_event_handler);

#ifdef DEBUG1
			printf("\t\tglobal_service_event_handler set to '%s'\n",global_service_event_handler);
#endif
		        }
		else if(!strcmp(variable,"admin_email")){
			if(strlen(value)>MAX_EMAIL_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Email address too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(macro_admin_email,value,sizeof(macro_admin_email));
			macro_admin_email[sizeof(macro_admin_email)-1]='\x0';
			strip(macro_admin_email);

#ifdef DEBUG1
			printf("\t\tmacro_admin_email set to '%s'\n",macro_admin_email);
#endif
		        }
		else if(!strcmp(variable,"admin_pager")){
			if(strlen(value)>MAX_PAGER_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Pager number/address too long)",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strncpy(macro_admin_pager,value,sizeof(macro_admin_pager));
			macro_admin_pager[sizeof(macro_admin_pager)-1]='\x0';
			strip(macro_admin_pager);

#ifdef DEBUG1
			printf("\t\tmacro_admin_pager set to '%s'\n",macro_admin_pager);
#endif
		        }
		else if(!strcmp(variable,"log_level")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'2'){
				printf("Error in main configuration file '%s' - Line %d (Invalid value for log_level)",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strip(value);
			log_level=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_level set to %d\n",log_level);
#endif
			}
		else if(!strcmp(variable,"use_syslog")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file '%s' - Line %d (Invalid value for use_syslog)",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strip(value);
			use_syslog=atoi(value);

#ifdef DEBUG1
			printf("\t\tuse_syslog set to %d\n",use_syslog);
#endif
			}
		else if(!strcmp(variable,"syslog_level")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'2'){
				printf("Error in main configuration file '%s' - Line %d (Invalid value for syslog_level)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strip(value);
			syslog_level=atoi(value);

#ifdef DEBUG1
			printf("\t\tsyslog_level set to %d\n",syslog_level);
#endif
			}
		else if(!strcmp(variable,"log_notifications")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_notifications)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_notifications=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_notifications set to %d\n",log_notifications);
#endif
		        }
		else if(!strcmp(variable,"log_service_retries")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_service_retries)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_service_retries=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_service_retries set to %d\n",log_service_retries);
#endif
		        }
		else if(!strcmp(variable,"log_host_retries")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_host_retries)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_host_retries=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_host_retries set to %d\n",log_host_retries);
#endif
		        }
		else if(!strcmp(variable,"log_event_handlers")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for log_event_handlers)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			log_event_handlers=atoi(value);

#ifdef DEBUG1
			printf("\t\tlog_event_handlers set to %d\n",log_event_handlers);
#endif
		        }
		else if(!strcmp(variable,"use_agressive_host_checking")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for use_agressive_host_checking)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			use_agressive_host_checking=atoi(value);

#ifdef DEBUG1
			printf("\t\tuse_agressive_host_checking set to %d\n",use_agressive_host_checking);
#endif
		        }
		else if(!strcmp(variable,"log_rotation_method")){
			if(!strcmp(value,"n"))
				log_rotation_method=LOG_ROTATION_NONE;
			else if(!strcmp(value,"h"))
				log_rotation_method=LOG_ROTATION_HOURLY;
			else if(!strcmp(value,"d"))
				log_rotation_method=LOG_ROTATION_DAILY;
			else if(!strcmp(value,"w"))
				log_rotation_method=LOG_ROTATION_WEEKLY;
			else if(!strcmp(value,"m"))
				log_rotation_method=LOG_ROTATION_MONTHLY;
			else{
				printf("Error in main configuration file %s - Line %d (Invalid value for log_rotation_method)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

#ifdef DEBUG1
			printf("\t\tlog_rotation_method set to %d\n",log_rotation_method);
#endif
		        }
		else if(!strcmp(variable,"log_archive_path")){
			if(strlen(value)>MAX_FILENAME_LENGTH-1){
				printf("Error in main configuration file '%s' - Line %d (Log archive path too long)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
				}

			strncpy(log_archive_path,value,sizeof(log_archive_path));
			log_archive_path[sizeof(log_archive_path)-1]='\x0';
			strip(log_archive_path);

#ifdef DEBUG1
			printf("\t\tlog_archive_path set to '%s'\n",log_archive_path);
#endif
			}
		else if(!strcmp(variable,"program_mode")){
			if(!strcmp(value,"s"))
				program_mode=STANDBY_MODE;
			else
				program_mode=ACTIVE_MODE;

#ifdef DEBUG1
		printf("\t\tprogram_mode set to %s\n",(program_mode==ACTIVE_MODE)?"ACTIVE":"STANDBY");
#endif
		        }
		else if(!strcmp(variable,"inter_check_delay_method")){
			if(!strcmp(value,"n"))
				inter_check_delay_method=ICD_NONE;
			else if(!strcmp(value,"d"))
				inter_check_delay_method=ICD_DUMB;
			else if(!strcmp(value,"s"))
				inter_check_delay_method=ICD_SMART;
			else{
				printf("Error in main configuration file %s - Line %d (Invalid value for inter_check_delay_method)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

#ifdef DEBUG1
			printf("\t\tinter_check_delay_method set to %d\n",inter_check_delay_method);
#endif
		        }
		else if(!strcmp(variable,"service_interleave_factor")){
			if(!strcmp(value,"s"))
				interleave_factor_method=ILF_SMART;
			else{
				interleave_factor_method=ILF_USER;
				interleave_factor=atoi(value);
				if(interleave_factor<1)
					interleave_factor=1;
			        }
		        }
		else if(!strcmp(variable,"max_concurrent_checks")){
			strip(value);
			max_parallel_service_checks=atoi(value);
			if(max_parallel_service_checks<1)
				max_parallel_service_checks=1;

#ifdef DEBUG1
			printf("\t\tmax_parallel_service_checks set to %d\n",max_parallel_service_checks);
#endif
		        }
		else if(!strcmp(variable,"service_reaper_frequency")){
			strip(value);
			service_check_reaper_interval=atoi(value);
			if(service_check_reaper_interval<1)
				service_check_reaper_interval=DEFAULT_SERVICE_REAPER_INTERVAL;

#ifdef DEBUG1
			printf("\t\tservice_check_reaper_interval set to %d\n",service_check_reaper_interval);
#endif
		        }
		else if(!strcmp(variable,"sleep_time")){
			strip(value);
			sleep_time=atoi(value);
			if(sleep_time<1)
				sleep_time=DEFAULT_SLEEP_TIME;

#ifdef DEBUG1
			printf("\t\tsleep_time set to %d\n",sleep_time);
#endif
		        }
		else if(!strcmp(variable,"interval_length")){
			strip(value);
			interval_length=atoi(value);
			if(interval_length<1)
				interval_length=DEFAULT_INTERVAL_LENGTH;

#ifdef DEBUG1
			printf("\t\tinterval_length set to %d\n",interval_length);
#endif
		        }
		else if(!strcmp(variable,"zombie_cleanup_interval")){
			strip(value);
			zombie_cleanup_interval=atoi(value);
			if(interval_length<1)
				zombie_cleanup_interval=DEFAULT_ZOMBIE_CLEANUP_INTERVAL;

#ifdef DEBUG1
			printf("\t\tzombie_cleanup_interval set to %d\n",zombie_cleanup_interval);
#endif
		        }
		else if(!strcmp(variable,"check_external_commands")){
			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
				printf("Error in main configuration file %s - Line %d (Invalid value for check_external_commands)\n",config_file,current_line);
				fclose(fp);
				return ERROR;
			        }

			strip(value);
			check_external_commands=atoi(value);

#ifdef DEBUG1
			printf("\t\tcheck_external_commands set to %d\n",check_external_commands);
#endif
		        }
		else if(!strcmp(variable,"command_check_interval")){
			strip(value);
			command_check_interval=atoi(value);
			if(command_check_interval<1)
				command_check_interval=DEFAULT_COMMAND_CHECK_INTERVAL;

#ifdef DEBUG1
			printf("\t\tcommand_check_interval set to %d\n",command_check_interval);
#endif
		        }
		else{

			printf("Error in main configuration file '%s' - Line %d (Unknown variable)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		}

	fclose(fp);

	/* make sure a log file has been specified */
	strip(log_file);
	if(!strcmp(log_file,"")){
		printf("Error: Log file is not specified in main config file '%s'!\n",config_file);
		return ERROR;
		}

#ifdef DEBUG0
	printf("read_main_config() end\n");
#endif

	return OK;
	}


/* process the host/contact/command/service configuration file */
int read_config_file(char *config_file){
	char input[MAX_INPUT_BUFFER];
	char variable[MAX_INPUT_BUFFER];
	char value[MAX_INPUT_BUFFER];
	char vartype[MAX_VAR_TYPE];
	char varname[MAX_VAR_NAME];
	FILE *fp;
	int current_line=0;
	int result;

#ifdef DEBUG0
	printf("read_config_file() start\n");
	printf("\tConfig file: %s\n",config_file);
#endif

	fp=fopen(config_file,"r");
	if(fp==NULL){
		printf("Error: Cannot open host configuration file '%s' for reading!\n",config_file);
		return ERROR;
		}

	/* read all lines from the config file */
	for(current_line=1,fgets(input,MAX_INPUT_BUFFER-1,fp);!feof(fp);current_line++,fgets(input,MAX_INPUT_BUFFER-1,fp)){

		/* skip blank lines and comments */
		if(input[0]=='#' || input[0]=='\x0' || input[0]=='\n' || input[0]=='\r')
			continue;

#ifdef DEBUG1
		printf("\tLine %d Raw Input: %s",current_line,input);
#endif

		strip(input);

#ifdef DEBUG1
		printf("\tLine %d Stripped Input: %s\n",current_line,input);
#endif

		result=parse_input(input,variable,value,sizeof(variable),sizeof(value));
		if(result!=OK){
			printf("Error in host configuration file '%s' - Line %d (Could not parse input)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

#ifdef DEBUG1
		printf("\tParsed Input: Variable=%s, Value=%s\n",variable,value);
#endif

		result=parse_variable(variable,vartype,varname,sizeof(vartype),sizeof(varname));
		if(result!=OK){
			printf("Error in host configuration file '%s' - Line %d (Could not parse variable)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

#ifdef DEBUG1
		printf("\tVar Type: %s\n",vartype);
		printf("\tVar Name: %s\n",varname);
#endif

		/* decide how to process the variable ...*/

		/* this is a TIMEPERIOD entry */
		if(!strcmp(vartype,"timeperiod"))
			result=add_timeperiod(varname,value,current_line);

		/* this is a HOST entry */
		else if(!strcmp(vartype,"host"))
			result=add_host(varname,value,current_line);

		/* this is a HOSTGROUP entry */
		else if(!strcmp(vartype,"hostgroup"))
			result=add_hostgroup(varname,value,current_line);

		/* this is a CONTACT entry */
		else if(!strcmp(vartype,"contact"))
			result=add_contact(varname,value,current_line);

		/* this is a CONTACTGROUP entry */
		else if(!strcmp(vartype,"contactgroup"))
			result=add_contactgroup(varname,value,current_line);

		/* this is a SERVICE entry */
		else if(!strcmp(vartype,"service"))
			result=add_service(varname,value,current_line);

		/* this is a COMMAND entry */
		else if(!strcmp(vartype,"command"))
			result=add_command(varname,value,current_line);

		/* else we don't know what the hell it is */
		else{

			printf("Error in host configuration file '%s' - Line %d (Unknown variable type)\n",config_file,current_line);
			fclose(fp);
			return ERROR;
			}

		/* check the result code after attempting to add the host, contact, etc. */
		if(result!=OK){
			fclose(fp);
			return ERROR;
			}
		}

	fclose(fp);

#ifdef DEBUG0
	printf("read_config_file() end\n");
#endif

	return OK;
	}


/* parse the raw input line and get the variable and value */
int parse_input(char *input, char *variable, char *value, int maxvarlength, int maxvallength){
	char *temp;

#ifdef DEBUG0
	printf("parse_input() start\n");
#endif

	/* get the variable */
	temp=my_strtok(input,"=");

	/* if it doesn't exist, return an error */
	if(temp==NULL){

#ifdef DEBUG
		printf("\tVariable is NULL\n");
#endif

		return ERROR;
		}

	/* else the variable is good */
	else{
		strncpy(variable,temp,maxvarlength);
		variable[maxvarlength-1]='\x0';
	        }

	/* get the value */
	/* Note: token changed from "=" to "\x0" to fix problem with equal signs in service check commands (0.0.2p1) */
	temp=my_strtok(NULL,"\x0");

	/* if the value doesn't exist, return an error */
	if(temp==NULL){

#ifdef DEBUG2
		printf("\tValue is NULL\n");
#endif

		return ERROR;
		}

	/* else the value is good */
	else{
		strncpy(value,temp,maxvallength);
		value[maxvallength-1]='\x0';
	        }

	strip(variable);
	strip(value);

#ifdef DEBUG0
	printf("parse_input() end\n");
#endif

	return OK;
	}


/* parse the variable name */
int parse_variable(char *varstring, char *vartype, char *varname, int maxtypelength, int maxnamelength){
	char *temp;

#ifdef DEBUG0
	printf("parse_variable() start\n");
#endif

	/* get the variable type */
	temp=my_strtok(varstring,"[");

	/* return error if its empty */
	if(temp==NULL){

#ifdef DEBUG2
		printf("\tVariable type is NULL\n");
#endif

		return ERROR;
		}

	/* return error if var type is too long */
	if(strlen(temp) > (maxtypelength-1)){

#ifdef DEBUG2
		printf("\tVariable type '%s' is too long\n",temp);
#endif

		return ERROR;
                }

	/* else its good */
	strncpy(vartype,temp,maxtypelength);
	vartype[maxtypelength-1]='\x0';

	/* now get the name of the variable */
	temp=my_strtok(NULL,"]");

	/* return error if its empty */
	if(temp==NULL){

#ifdef DEBUG2
		printf("\tVariable name is NULL\n");
#endif

		return ERROR;
		}

	/* return error if var name is too long */
	if(strlen(temp) > (maxnamelength-1)){

#ifdef DEBUG2
		printf("\tVariable name '%s' is too long\n",temp);
#endif

		return ERROR;
                }

	/* else its good */
	strncpy(varname,temp,maxnamelength);
	varname[maxnamelength-1]='\x0';

	strip(vartype);
	strip(varname);

#ifdef DEBUG0
	printf("parse_variable() end\n");
#endif

	return OK;
	}






/****************************************************************/
/**************** CONFIG VERIFICATION FUNCTIONS *****************/
/****************************************************************/

/* do a pre-flight check to make sure object relationships make sense */
int pre_flight_check(void){
	contact *temp_contact;
	commandsmember *temp_commandsmember;
	contactgroup *temp_contactgroup;
	contactgroupmember *temp_contactgroupmember;
	host *temp_host;
	hostgroup *temp_hostgroup;
	hostgroupmember *temp_hostgroupmember;
	service *temp_service;
	command *temp_command;
	timeperiod *temp_timeperiod;
	char temp_buffer[MAX_INPUT_BUFFER];
	char *temp_command_name="";
	char *temp_periodname="";
	char contactgroup_names[MAX_CONTACTGROUPS_LENGTH];
	char *current_contactgroup;
	int found;

#ifdef DEBUG0
	printf("pre_flight_check() start\n");
#endif



	/*****************************************/
	/* check each host group...              */
	/*****************************************/
	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

		/* check all group members */
		for(temp_hostgroupmember=temp_hostgroup->members;temp_hostgroupmember!=NULL;temp_hostgroupmember=temp_hostgroupmember->next){

			temp_host=find_host(temp_hostgroupmember->host_name,NULL);
			if(temp_host==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Host '%s' specified in host group '%s' is not defined anywhere!\n",temp_hostgroupmember->host_name,temp_hostgroup->group_name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }

		        }

		found=FALSE;

		/* check all contact groups */
		strcpy(contactgroup_names,temp_hostgroup->contact_groups);
		for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
			temp_contactgroup=find_contactgroup(current_contactgroup,NULL);

			if(temp_contactgroup==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Contact group '%s' specified in host group '%s' is not defined anywhere!\n",current_contactgroup,temp_hostgroup->group_name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
			}
		}

#ifdef DEBUG1
	printf("\tCompleted hostgroup verification checks\n");
#endif


	/*****************************************/
	/* check each contact group...           */
	/*****************************************/
	for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){

		found=FALSE;

		/* make sure each contactgroup is used in at least one hostgroup or service */
		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
			strcpy(contactgroup_names,temp_hostgroup->contact_groups);
			for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
				if(!strcmp(temp_contactgroup->group_name,current_contactgroup)){
					found=TRUE;
					break;
				        }
			        }
			if(found==TRUE)
				break;
			}
		if(found==FALSE){
			for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
				strcpy(contactgroup_names,temp_service->contact_groups);
				for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
					if(!strcmp(temp_contactgroup->group_name,current_contactgroup)){
						found=TRUE;
						break;
				                }
			                 }
				if(found==TRUE)
					break;
			        }
		        }

		/* we couldn't find a hostgroup or service */
		if(found==FALSE){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Contact group '%s' is not used in any hostgroup or service definition!\n",temp_contactgroup->group_name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			}

		/* check all the group members */
		for(temp_contactgroupmember=temp_contactgroup->members;temp_contactgroupmember!=NULL;temp_contactgroupmember=temp_contactgroupmember->next){

			temp_contact=find_contact(temp_contactgroupmember->contact_name,NULL);
			if(temp_contact==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Contact '%s' specified in contact group '%s' is not defined anywhere!\n",temp_contactgroupmember->contact_name,temp_contactgroup->group_name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }

		        }

		}

#ifdef DEBUG1
	printf("\tCompleted contact group verification checks\n");
#endif



	/*****************************************/
	/* check each service...                 */
	/*****************************************/
	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

		found=FALSE;

		/* check for a valid host */
		for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

			/* we found the host! */
			if(!strcmp(temp_service->host_name,temp_host->name)){
				found=TRUE;
				break;
				}
			}

		/* we couldn't find an associated host! */
		if(found==FALSE){
			snprintf(temp_buffer,sizeof(temp_buffer),"Error: Host '%s' specified in service '%s' not defined anywhere!\n",temp_service->host_name,temp_service->description);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
			}

		/* check the event handler command */
		temp_command_name=temp_service->event_handler;
		if(strcmp(temp_command_name,"")){
			temp_command=find_command(temp_command_name,NULL);
			if(temp_command==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Event handler command '%s' specified in service '%s' for host '%s' not defined anywhere\n",temp_command_name,temp_service->description,temp_service->host_name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check the service check_command */
		strncpy(temp_buffer,temp_service->command,sizeof(temp_buffer));
		temp_buffer[sizeof(temp_buffer)-1]='\x0';

		/* raw command lines shouldn't be checked */
		if(temp_buffer[0]=='"')
			continue;

		/* get the command name, leave any arguments behind */
		temp_command_name=my_strtok(temp_buffer,"!");

		temp_command=find_command(temp_command_name,NULL);
		if(temp_command==NULL){
			snprintf(temp_buffer,sizeof(temp_buffer),"Error: Service check command '%s' specified in service '%s' for host '%s' not defined anywhere!\n",temp_command_name,temp_service->description,temp_service->host_name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }		

		/* check for sane recovery options */
		if(temp_service->notify_on_recovery && !temp_service->notify_on_warning && !temp_service->notify_on_critical){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Recovery notification option in service '%s' for host '%s' doesn't make any sense - specify warning and/or critical options as well\n",temp_service->description,temp_service->host_name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		/* reset the found flag */
		found=FALSE;

		/* check for valid contactgroups */
		strcpy(contactgroup_names,temp_service->contact_groups);
		for(current_contactgroup=my_strtok(contactgroup_names,",");current_contactgroup!=NULL;current_contactgroup=my_strtok(NULL,",")){
			temp_contactgroup=find_contactgroup(current_contactgroup,NULL);

			if(temp_contactgroup==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Contact group '%s' specified in service '%s' for host '%s' is not defined anywhere!\n",current_contactgroup,temp_service->description,temp_service->host_name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
			}

		/* verify service check timeperiod */
		temp_periodname=temp_service->check_period;

		if(!strcmp(temp_periodname,"")){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Service '%s' has no check time period defined!\n",temp_service->description);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Check period '%s' specified for service '%s' on host '%s' is not defined anywhere!\n",temp_periodname,temp_service->description,temp_service->host_name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check service notification timeperiod */
		temp_periodname=temp_service->notification_period;

		if(!strcmp(temp_periodname,"")){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Service '%s' on host '%s' has no notification time period defined!\n",temp_service->description,temp_service->host_name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Notification period '%s' specified for service '%s' on host '%s' is not defined anywhere!\n",temp_periodname,temp_service->description,temp_service->host_name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }
	        }


#ifdef DEBUG1
	printf("\tCompleted service verification checks\n");
#endif



	/*****************************************/
	/* check all hosts...                    */
	/*****************************************/
	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){

		found=FALSE;

		/* make sure each host has at least one service associated with it */
		for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){

			/* we found a service! */
			if(!strcmp(temp_host->name,temp_service->host_name)){
				found=TRUE;
				break;
				}
			}

		/* we couldn't find a service associated with this host! */
		if(found==FALSE){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Host '%s' has no services associated with it!\n",temp_host->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			}

		found=FALSE;

		/* make sure each host is a member of at least one hostgroup */
		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
			temp_hostgroupmember=find_hostgroupmember(temp_host->name,temp_hostgroup,NULL);
			if(temp_hostgroupmember!=NULL){
				found=TRUE;
				break;
			        }
		        }

		/* we couldn't find the host in any host groups */
		if(found==FALSE){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Host '%s' is not a member of any host groups!\n",temp_host->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		/* check the event handler command */
		temp_command_name=temp_host->event_handler;
		if(strcmp(temp_command_name,"")){
			temp_command=find_command(temp_command_name,NULL);
			if(temp_command==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Event handler command '%s' specified for host '%s' not defined anywhere\n",temp_command_name,temp_host->name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check the host check command */
		strcpy(temp_buffer,temp_host->host_check_command);

		/* hosts that don't have check commands defined shouldn't ever be checked... */
		if(strcmp(temp_buffer,"")){

			temp_command_name=&temp_buffer[0];
			temp_command=find_command(temp_command_name,NULL);
			if(temp_command==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Host check command '%s' specified for host '%s' is not defined anywhere!\n",temp_command_name,temp_host->name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
		                }
		        }

		/* check notification timeperiod */
		temp_periodname=temp_host->notification_period;

		if(!strcmp(temp_periodname,"")){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Host '%s' has no notification time period defined!\n",temp_host->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Notification period '%s' specified for host '%s' is not defined anywhere!\n",temp_periodname,temp_host->name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check the parent host */
		if(strcmp(temp_host->parent_host,"")){

			if(find_host(temp_host->parent_host,NULL)==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Host '%s' does not have a valid parent host!\n",temp_host->name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
		                }
		        }

		/* check for sane recovery options */
		if(temp_host->notify_on_recovery && !temp_host->notify_on_down && !temp_host->notify_on_unreachable){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Recovery notification option in host '%s' doesn't make any sense - specify down and/or unreachable options as well\n",temp_host->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

	        }


#ifdef DEBUG1
	printf("\tCompleted host verification checks\n");
#endif


	/*****************************************/
	/* check all contacts...                 */
	/*****************************************/
	for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){

		/* check service notification commands */
		if(temp_contact->service_notification_commands==NULL){
			snprintf(temp_buffer,sizeof(temp_buffer),"Error: Contact '%s' has no service notification commands defined!\n",temp_contact->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }
		else for(temp_commandsmember=temp_contact->service_notification_commands;temp_commandsmember!=NULL;temp_commandsmember=temp_commandsmember->next){
		        temp_command=find_command(temp_commandsmember->command,NULL);
			if(temp_command==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Service notification command '%s' specified for contact '%s' is not defined anywhere!\n",temp_commandsmember->command,temp_contact->name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check host notification commands */
		if(temp_contact->host_notification_commands==NULL){
			snprintf(temp_buffer,sizeof(temp_buffer),"Error: Contact '%s' has no host notification commands defined!\n",temp_contact->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }
		else for(temp_commandsmember=temp_contact->host_notification_commands;temp_commandsmember!=NULL;temp_commandsmember=temp_commandsmember->next){
			temp_command=find_command(temp_commandsmember->command,NULL);
			if(temp_command==NULL){
				sprintf(temp_buffer,"Error: Host notification command '%s' specified for contact '%s' is not defined anywhere!\n",temp_commandsmember->command,temp_contact->name);
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
	                }

		/* check service notification timeperiod */
		temp_periodname=temp_contact->service_notification_period;

		if(!strcmp(temp_periodname,"")){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Contact '%s' has no service notification time period defined!\n",temp_contact->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Service notification period '%s' specified for contact '%s' is not defined anywhere!\n",temp_periodname,temp_contact->name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		/* check host notification timeperiod */
		temp_periodname=temp_contact->host_notification_period;

		if(!strcmp(temp_periodname,"")){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Contact '%s' has no host notification time period defined!\n",temp_contact->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		else{
		        temp_timeperiod=find_timeperiod(temp_periodname,NULL);
			if(temp_timeperiod==NULL){
				snprintf(temp_buffer,sizeof(temp_buffer),"Error: Host notification period '%s' specified for contact '%s' is not defined anywhere!\n",temp_periodname,temp_contact->name);
				temp_buffer[sizeof(temp_buffer)-1]='\x0';
				printf("%s",temp_buffer);
				write_to_log(temp_buffer);
				return ERROR;
			        }
		        }

		found=FALSE;

		/* make sure the contact belongs to at least one contact group */
		for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){
			temp_contactgroupmember=find_contactgroupmember(temp_contact->name,temp_contactgroup,NULL);
			if(temp_contactgroupmember!=NULL){
				found=TRUE;
				break;
			        }
		        }

		/* we couldn't find the contact in any contact groups */
		if(found==FALSE){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Contact '%s' is not a member of any contact groups!\n",temp_contact->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }
	
		/* check for sane host recovery options */
		if(temp_contact->notify_on_host_recovery && !temp_contact->notify_on_host_down && !temp_contact->notify_on_host_unreachable){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Host recovery notification option for contact '%s' doesn't make any sense - specify down and/or unreachable options as well\n",temp_contact->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }

		/* check for sane service recovery options */
		if(temp_contact->notify_on_service_recovery && !temp_contact->notify_on_service_critical && !temp_contact->notify_on_service_warning){
			snprintf(temp_buffer,sizeof(temp_buffer),"Warning: Service recovery notification option for contact '%s' doesn't make any sense - specify critical and/or warning options as well\n",temp_contact->name);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted contact verification checks\n");
#endif

 
	/********************************************/
	/* check global event handler commands...   */
	/********************************************/
	if(strcmp(global_host_event_handler,"")){

	        temp_command=find_command(global_host_event_handler,NULL);
		if(temp_command==NULL){
			snprintf(temp_buffer,sizeof(temp_buffer),"Error: Global host event handler command '%s' is not defined anywhere!\n",global_host_event_handler);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }
	        }
	if(strcmp(global_service_event_handler,"")){

	        temp_command=find_command(global_service_event_handler,NULL);
		if(temp_command==NULL){
			snprintf(temp_buffer,sizeof(temp_buffer),"Error: Global service event handler command '%s' is not defined anywhere!\n",global_service_event_handler);
			temp_buffer[sizeof(temp_buffer)-1]='\x0';
			printf("%s",temp_buffer);
			write_to_log(temp_buffer);
			return ERROR;
		        }
	        }

#ifdef DEBUG1
	printf("\tCompleted global event handler command checks\n");
#endif

#ifdef DEBUG0
	printf("pre_flight_check() end\n");
#endif

	return OK;
	}

