/*****************************************************************************
 *
 * IPC.C - IPC Functions for NetSaint
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified:   02-14-2000
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/


#include "../common/config.h"
#include "../common/common.h"

#include "netsaint.h"


extern int ipc_message_queue;



/******************************************************************/
/************************* IPC FUNCTIONS **************************/
/******************************************************************/


/* initializes (creates) a message queue */
int init_message_queue(void){
	key_t key=-1;
        int queue_id;
	int x=0;

#ifdef DEBUG0
	printf("init_message_queue() start\n");
#endif

	/* make 10 attempts at creating a key */
	for(x=0;x<10;x++){
		
		/* create a unique key for use in message queue creation */
		key=ftok(".", (char)((int)'m'+x));

		/* if we were able to create the key, break out.. */
		if(key!=-1)
			break;
	        }

	if(key==-1)
		return -1;

        /* create the message queue */
        queue_id=msgget(key,IPC_CREAT|0660);

#ifdef DEBUG0
	printf("init_message_queue() end\n");
#endif

	return queue_id;
        }


/* removes a message queue */
void remove_message_queue(int queue_id){

#ifdef DEBUG0
	printf("remove_message_queue() start\n");
#endif

        /* remove the queue */
        msgctl(queue_id,IPC_RMID,0);

#ifdef DEBUG0
	printf("remove_message_queue() end\n");
#endif

	return;
	}


/* send a service message to a message queue */
int send_svc_message(int queue_id,service_message *message){
	int send_result;

#ifdef DEBUG0
	printf("send_svc_message() start\n");
#endif

        /* send a message to the queue */
        message->mtype=SERVICE_MESSAGE_ID;
        send_result=msgsnd(queue_id,(service_message *)message,sizeof(service_message)-sizeof(long),0);

#ifdef DEBUG0
	printf("send_svc_message() end\n");
#endif

	return send_result;
	}


/* reads a service message from a message queue */
int read_svc_message(int queue_id,service_message *message){
	int read_result;

#ifdef DEBUG0
	printf("read_svc_message() start\n");
#endif

	/* clear the buffer */
	bzero(message,sizeof(service_message));

        /* read a message from the queue */
        read_result=msgrcv(queue_id,(service_message *)message,sizeof(service_message)-sizeof(long),SERVICE_MESSAGE_ID,IPC_NOWAIT);

#ifdef DEBUG0
	printf("read_svc_message() end\n");
#endif

	return read_result;
	}

