/************************************************************************
 *
 * NOTIFICATIONS.C - NetSaint Notifications CGI
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 11-09-1999
 *
 * This CGI program will display the notification events for 
 * a given host or contact or for all contacts/hosts.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "getcgi.h"
#include "cgiutils.h"
#include "auth.h"
#include "lifo.h"

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_html_path[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char url_docs_path[MAX_FILENAME_LENGTH];

extern char html_table_bg_color[MAX_COLOR_LENGTH];
extern char html_table_fg_color[MAX_COLOR_LENGTH];
extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];
extern char html_table_bg_even_color[MAX_COLOR_LENGTH];

extern int    log_rotation_method;
extern time_t this_scheduled_log_rotation;
extern time_t last_scheduled_log_rotation;


#define FIND_HOST		1
#define FIND_CONTACT		2

#define MAX_QUERYNAME_LENGTH	256

#define SERVICE_NOTIFICATION	0
#define HOST_NOTIFICATION	1

#define SERVICE_NOTIFICATION_STRING 	"] SERVICE NOTIFICATION:"
#define HOST_NOTIFICATION_STRING	"] HOST NOTIFICATION:"

void get_notifications(int,char *);

void document_header(void);
void document_footer(void);
int process_cgivars(void);

authdata current_authdata;

char log_file_to_use[MAX_FILENAME_LENGTH];
int log_archive=0;

int find_type=FIND_HOST;
int find_all=FALSE;
char find_query[MAX_QUERYNAME_LENGTH]="";

int notification_options=NOTIFICATION_ALL;
int use_lifo=TRUE;


int main(void){
	int error=FALSE;
	int result=OK;
	time_t t;
	char date_time[48];

	error=process_cgivars();

	document_header();

	if(error==FALSE){

		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file '%s' for reading!\n",main_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_all_host_config_files(READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_HOSTS|READ_SERVICES);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open host configuration file(s) for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
                        }

		get_log_archive_to_use(log_archive,log_file_to_use,(int)sizeof(log_file_to_use));

		/* begin top table */
		printf("<p>\n");
		printf("<table border=0 width=100%%>\n");
		printf("<tr>\n");
		printf("<td align=left valign=top>\n");

		printf("<table border=0>\n");
		printf("<tr><td>\n");

		/* left column of the first row */
		printf("<table border=1>\n");
		printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
		printf("<font size=+2><b>Alert Notifications</b></font><br>\n");
		printf("<font size=-2>\n");
		time(&t);
		get_time_string(&t,date_time,(int)sizeof(date_time));
		printf("Last Updated: %s<BR>",date_time);
		printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
		printf("</font>\n");
		printf("</td></tr>\n");

		if(find_all==FALSE){
			printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
			printf("<font size=-2>\n");
			printf("<a href=\"%s?%s=all\">View notifications for all %s</a>\n",NOTIFICATIONS_CGI,(find_type==FIND_HOST)?"host":"contact",(find_type==FIND_HOST)?"hosts":"contacts");
			printf("</font>\n");
			printf("</td></tr>\n");
	                }

		printf("</table>\n");
		printf("</td>\n");

		printf("<td width=30></td>\n");

		/* middle column of top row */
		printf("<td align=left valign=center>\n");
		printf("<font size=+2><b>");
		if(find_all==TRUE)
			printf("Notifications For All %s\n",(find_type==FIND_HOST)?"Hosts":"Contacts");
		else
			printf("Notifications For %s \"%s\"",(find_type==FIND_HOST)?"Host":"Contact",find_query);
		printf("</b></font>\n");
		printf("</td>\n");

		printf("</tr>\n");
		printf("</table>\n");
		printf("</td>\n");


		/* right hand column of top row */
		printf("<td align=right valign=bottom>\n");

		printf("<table border=0>\n");
		printf("<form method=\"GET\" action=\"%s\">\n",NOTIFICATIONS_CGI);
		printf("<input type='hidden' name='%s' value='%s'>\n",(find_type==FIND_HOST)?"host":"contact",find_query);
		printf("<input type='hidden' name='archive' value='%d'>\n",log_archive);
		printf("<tr>\n");
		printf("<td align=left><font size=-2>Notification detail level for %s %s%s:</font></td>",(find_all==TRUE)?"all":"this",(find_type==FIND_HOST)?"host":"contact",(find_all==TRUE)?"s":"");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left><font size=-1><select name='type'>\n");
		printf("<option value=%d %s>All notifications\n",NOTIFICATION_ALL,(notification_options==NOTIFICATION_ALL)?"selected":"");
		printf("<option value=%d %s>All service notifications\n",NOTIFICATION_SERVICE_ALL,(notification_options==NOTIFICATION_SERVICE_ALL)?"selected":"");
		printf("<option value=%d %s>All host notifications\n",NOTIFICATION_HOST_ALL,(notification_options==NOTIFICATION_HOST_ALL)?"selected":"");
		printf("<option value=%d %s>Service warning\n",NOTIFICATION_SERVICE_WARNING,(notification_options==NOTIFICATION_SERVICE_WARNING)?"selected":"");
		printf("<option value=%d %s>Service unknown\n",NOTIFICATION_SERVICE_UNKNOWN,(notification_options==NOTIFICATION_SERVICE_UNKNOWN)?"selected":"");
		printf("<option value=%d %s>Service critical\n",NOTIFICATION_SERVICE_CRITICAL,(notification_options==NOTIFICATION_SERVICE_CRITICAL)?"selected":"");
		printf("<option value=%d %s>Service recovery\n",NOTIFICATION_SERVICE_RECOVERY,(notification_options==NOTIFICATION_SERVICE_RECOVERY)?"selected":"");
		printf("<option value=%d %s>Host down\n",NOTIFICATION_HOST_DOWN,(notification_options==NOTIFICATION_HOST_DOWN)?"selected":"");
		printf("<option value=%d %s>Host unreachable\n",NOTIFICATION_HOST_UNREACHABLE,(notification_options==NOTIFICATION_HOST_UNREACHABLE)?"selected":"");
		printf("<option value=%d %s>Host recovery\n",NOTIFICATION_HOST_RECOVERY,(notification_options==NOTIFICATION_HOST_RECOVERY)?"selected":"");
		printf("</select></font></td>\n");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left><font size=-2>Older Entries First:</font></td>\n");
		printf("<td></td>\n");
		printf("</tr>\n");
		printf("<tr>\n");
		printf("<td align=left valign=bottom><font size=-1><input type='checkbox' name='oldestfirst' %s></font></td>",(use_lifo==FALSE)?"checked":"");
		printf("<td align=left><font size=-1><input type='submit' value='Update'></font></td>\n");
		printf("</tr>\n");
		printf("</form>\n");
		printf("</table>\n");

		printf("</td>\n");

		/* end of top table */
		printf("</tr>\n");
		printf("</table>\n");
		printf("</p>\n");


		if(log_rotation_method!=LOG_ROTATION_NONE){
			printf("<p>\n");
			printf("<div align=\"center\">\n");
			printf("<table border=0>\n");
			printf("<tr>\n");
			printf("<td>\n");

			printf("<table border=0 cellspacing=20 cellpadding=0>\n");
			printf("<tr>\n");
			printf("<td align=center valign=bottom>\n");
			printf("<font size=-1>");
			if(log_archive==0){
				printf("<font size=-1 color=red>Latest Archive</font><br>");
				printf("<a href=\"%s?archive=1&%s=%s&type=%d%s\"><img src=\"%s%s\" border=0 alt=\"Latest Archive\"></a>",NOTIFICATIONS_CGI,(find_type==FIND_HOST)?"host":"contact",url_encode(find_query),notification_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,LEFT_ARROW_ICON);
			        }
			else{
				printf("<font size=-1 color=red>Earlier Archive</font><br>");
				printf("<a href=\"%s?archive=%d&%s=%s&type=%d%s\"><img src=\"%s%s\" border=0 alt=\"Earlier Archive\"></a>",NOTIFICATIONS_CGI,log_archive+1,(find_type==FIND_HOST)?"host":"contact",url_encode(find_query),notification_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,LEFT_ARROW_ICON);
			        }
			printf("</font>");
			printf("</td>\n");

			if(log_archive!=0){
				printf("<td align=center>\n");
				get_time_string((unsigned long*)&last_scheduled_log_rotation,date_time,(int)sizeof(date_time));
				printf("<font size=-2><strong>%s</strong>",date_time);
				printf("<br>to<br>");
				get_time_string((unsigned long*)&this_scheduled_log_rotation,date_time,(int)sizeof(date_time));
				printf("<strong>%s</strong></font>",date_time);

				printf("</td>\n");
				printf("<td align=center valign=bottom>\n");

				printf("<font size=-1>");
				if(log_archive==1){
					printf("<font size=-1 color=red>Current Notifications</font><br>");
					printf("<a href=\"%s?%s=%s&type=%d%s\"><IMG SRC=\"%s%s\" border=0 alt=\"Current Notifications\"></a>",NOTIFICATIONS_CGI,(find_type==FIND_HOST)?"host":"contact",url_encode(find_query),notification_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,RIGHT_ARROW_ICON);
				        }
				else{
					printf("<font size=-1 color=red>More Recent Archive</font><br>");
					printf("<a href=\"%s?archive=%d&%s=%s&type=%d%s\"><img src=\"%s%s\" border=0 alt=\"More Recent Archive\"></a>",NOTIFICATIONS_CGI,log_archive-1,(find_type==FIND_HOST)?"host":"contact",url_encode(find_query),notification_options,(use_lifo==FALSE)?"&oldestfirst":"",url_images_path,RIGHT_ARROW_ICON);
				        }
				printf("</font>");
				printf("</td>\n");
		                }

			printf("</tr>\n");
			printf("</table>\n");
	
			printf("</td>\n");
			printf("</tr>\n");
			printf("</table>\n");
			printf("</div>\n");
			printf("</p>\n");
	                }

		/* get authentication information */
		get_authentication_information(&current_authdata);

		get_notifications(find_type,find_query);
	        }

	else{
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied");
		printf("</strong></font></div></p>\n"); 
	        }

	document_footer();
	
	return OK;
        }



void document_header(void){
	char date_time[48];
	time_t current_time;
	time_t expire_time;


	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");

	time(&current_time);
	get_expire_time_string(&current_time,date_time,(int)sizeof(date_time));
	printf("Last-Modified: %s\n",date_time);

	expire_time=(time_t)0L;
	get_expire_time_string(&expire_time,date_time,(int)sizeof(date_time));
	printf("Expires: %s\n",date_time);

	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<title>\n");
	printf("Alert Notifications\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body bgcolor=\"%s\">\n",HTML_BACKGROUND_COLOR);

	return;
        }



void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			find_type=FIND_HOST;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_QUERYNAME_LENGTH-1){
				strcpy(find_query,"all");
			        }
			else{
				strncpy(find_query,variables[x],sizeof(find_query));
				find_query[sizeof(find_query)-1]='\x0';
			        }

			if(!strcmp(find_query,"all"))
				find_all=TRUE;
			else
				find_all=FALSE;
		        }
	
		/* we found the contact argument */
		else if(!strcmp(variables[x],"contact")){
			find_type=FIND_CONTACT;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_QUERYNAME_LENGTH-1){
				strcpy(find_query,"all");
			        }
			else{
				strncpy(find_query,variables[x],sizeof(find_query));
				find_query[sizeof(find_query)-1]='\x0';
			        }

			if(!strcmp(find_query,"all"))
				find_all=TRUE;
			else
				find_all=FALSE;
		        }
	
		/* we found the notification type argument */
		else if(!strcmp(variables[x],"type")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			notification_options=atoi(variables[x]);
		        }
	
		/* we found the log archive argument */
		else if(!strcmp(variables[x],"archive")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			log_archive=atoi(variables[x]);
		        }
	
		/* we found the order argument */
		else if(!strcmp(variables[x],"oldestfirst")){
			use_lifo=FALSE;
		        }

	        }

	return error;
        }



void get_notifications(int find_type,char *varvalue){
	FILE *fp=NULL;
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	char date_time[MAX_INPUT_BUFFER];
	char alert_level[MAX_INPUT_BUFFER];
	char contact_name[MAX_INPUT_BUFFER];
	char service_name[MAX_INPUT_BUFFER];
	char host_name[MAX_INPUT_BUFFER];
	char method_name[MAX_INPUT_BUFFER];
	char alert_color[MAX_INPUT_BUFFER];
	int show_entry;
	int total_notifications;
	int notification_type=SERVICE_NOTIFICATION;
	int notification_detail_type=NOTIFICATION_SERVICE_CRITICAL;
	int odd=0;
	time_t t;
	host *temp_host;
	service *temp_service;
	int result;

	if(use_lifo==TRUE){
		result=read_file_into_lifo(log_file_to_use);
		if(result!=LIFO_OK){
			if(result==LIFO_ERROR_MEMORY){
				printf("<p>\n");
				printf("<div align=\"center\">\n");
				printf("<font color=\"red\"><strong>");
				printf("<blink>Not enough memory to reverse log file - displaying notifications in natural order...</blink>");
				printf("</strong></font>\n");
				printf("</div>\n");
				printf("</p>\n");
			        }
			else if(result==LIFO_ERROR_FILE){
				printf("<p>\n");
				printf("<div align=\"center\"><font color=\"red\"><strong>");
				printf("Error: Cannot open log file '%s' for reading!",log_file_to_use);
				printf("</strong></font></div>\n");
				printf("</p>\n");
				return;
			        }
			use_lifo=FALSE;
		        }
	        }

	if(use_lifo==FALSE){

		fp=fopen(log_file_to_use,"r");
		if(fp==NULL){
			printf("<p>\n");
			printf("<div align=\"center\"><font color=\"red\"><strong>");
			printf("Error: Cannot open log file '%s' for reading!",log_file_to_use);
			printf("</strong></font></div>\n");
			printf("</p>\n");
			return;
		        }
	        }

	printf("<p>\n");
	printf("<div align=\"center\">\n");
	printf("<table border=0 cellspacing=3 cellpadding=3>\n");
	printf("<tr>\n");
	printf("<th align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host</FONT></th>\n",html_table_bg_color,html_table_fg_color);
	printf("<th align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Service</FONT></th>\n",html_table_bg_color,html_table_fg_color);
	printf("<th align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Status</FONT></th>\n",html_table_bg_color,html_table_fg_color);
	printf("<th align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Date/Time</FONT></th>\n",html_table_bg_color,html_table_fg_color);
	printf("<th align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Contact</FONT></th>\n",html_table_bg_color,html_table_fg_color);
	printf("<th align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Notification Command</FONT></th>\n",html_table_bg_color,html_table_fg_color);
	printf("<th align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Information</FONT></th>\n",html_table_bg_color,html_table_fg_color);
	printf("</tr>\n");

	total_notifications=0;
  
	while(1){
    
		if(use_lifo==TRUE){
			if(pop_lifo(input_buffer,(int)sizeof(input_buffer))!=LIFO_OK)
				break;
		        }
		else{
			fgets(input_buffer,(int)(sizeof(input_buffer)-1),fp);
			if(feof(fp))
				break;
		        }

		/* see if this line contains the notification event string */
		if(strstr(input_buffer,HOST_NOTIFICATION_STRING)||strstr(input_buffer,SERVICE_NOTIFICATION_STRING)){

			if(strstr(input_buffer,HOST_NOTIFICATION_STRING))
				notification_type=HOST_NOTIFICATION;
			else
				notification_type=SERVICE_NOTIFICATION;
      
			/* get the date/time */
			temp_buffer=(char *)strtok(input_buffer,"]");
			t=(time_t)(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);
			get_time_string(&t,date_time,(int)sizeof(date_time));
			strip(date_time);

			/* get the contact name */
			temp_buffer=(char *)strtok(NULL,":");
			temp_buffer=(char *)strtok(NULL,";");
			snprintf(contact_name,sizeof(contact_name),"%s",(temp_buffer==NULL)?"":temp_buffer+1);
			contact_name[sizeof(contact_name)-1]='\x0';

			/* get the host name */
			temp_buffer=(char *)strtok(NULL,";");
			snprintf(host_name,sizeof(host_name),"%s",(temp_buffer==NULL)?"":temp_buffer);
			host_name[sizeof(host_name)]='\x0';

			/* get the service name */
			if(notification_type==SERVICE_NOTIFICATION){
				temp_buffer=(char *)strtok(NULL,";");
				snprintf(service_name,sizeof(service_name),"%s",(temp_buffer==NULL)?"":temp_buffer);
				service_name[sizeof(service_name)-1]='\x0';
			        }

			/* get the alert level */
			temp_buffer=(char *)strtok(NULL,";");
			snprintf(alert_level,sizeof(alert_level),"%s",(temp_buffer==NULL)?"":temp_buffer);
			alert_level[sizeof(alert_level)-1]='\x0';

			if(notification_type==SERVICE_NOTIFICATION){

				if(!strcmp(alert_level,"CRITICAL")){
					strncpy(alert_color,CRITICAL_COLOR,sizeof(alert_color));
					alert_color[sizeof(alert_color)-1]='\x0';
					notification_detail_type=NOTIFICATION_SERVICE_CRITICAL;
				        }
				else if(!strcmp(alert_level,"WARNING")){
					strncpy(alert_color,WARNING_COLOR,sizeof(alert_color));
					alert_color[sizeof(alert_color)-1]='\x0';
					notification_detail_type=NOTIFICATION_SERVICE_WARNING;
				        }
				else if(!strcmp(alert_level,"RECOVERY")){
					strncpy(alert_color,RECOVERY_COLOR,sizeof(alert_color));
					alert_color[sizeof(alert_color)-1]='\x0';
					notification_detail_type=NOTIFICATION_SERVICE_RECOVERY;
				        }
				else{
					strncpy(alert_color,UNKNOWN_COLOR,sizeof(alert_color));
					alert_color[sizeof(alert_color)-1]='\x0';
					notification_detail_type=NOTIFICATION_SERVICE_UNKNOWN;
				        }
			        }

			else{

				if(!strcmp(alert_level,"DOWN")){
					strncpy(alert_color,HOST_DOWN_COLOR,sizeof(alert_color));
					alert_color[sizeof(alert_color)-1]='\x0';
					strncpy(alert_level,"HOST DOWN",sizeof(alert_level));
					alert_level[sizeof(alert_level)-1]='\x0';
					notification_detail_type=NOTIFICATION_HOST_DOWN;
				        }
				if(!strcmp(alert_level,"UNREACHABLE")){
					strncpy(alert_color,HOST_UNREACHABLE_COLOR,sizeof(alert_color));
					alert_color[sizeof(alert_color)-1]='\x0';
					strncpy(alert_level,"HOST UNREACHABLE",sizeof(alert_level));
					alert_level[sizeof(alert_level)-1]='\x0';
					notification_detail_type=NOTIFICATION_HOST_UNREACHABLE;
				        }
				else if(!strcmp(alert_level,"RECOVERY")){
					strncpy(alert_color,HOST_RECOVERY_COLOR,sizeof(alert_color));
					alert_color[sizeof(alert_color)-1]='\x0';
					strncpy(alert_level,"HOST RECOVERY",sizeof(alert_level));
					alert_level[sizeof(alert_level)-1]='\x0';
					notification_detail_type=NOTIFICATION_HOST_RECOVERY;
				        }
			        }

			/* get the method name */
			temp_buffer=(char *)strtok(NULL,";");
			snprintf(method_name,sizeof(method_name),"%s",(temp_buffer==NULL)?"":temp_buffer);
			method_name[sizeof(method_name)-1]='\x0';

			/* move to the informational message */
			temp_buffer=strtok(NULL,";");

			show_entry=FALSE;
      
			/* if we're searching by contact, filter out unwanted contact */
			if(find_type==FIND_CONTACT){
				if(find_all==TRUE)
					show_entry=TRUE;
				else if(!strcmp(varvalue,contact_name))
					show_entry=TRUE;
			        }

			if(find_type==FIND_HOST){
				if(find_all==TRUE)
					show_entry=TRUE;
				else if(!strcmp(varvalue,host_name))
					show_entry=TRUE;
			        }

			if(show_entry==TRUE){
				if(notification_options==NOTIFICATION_ALL)
					show_entry=TRUE;
				else if(notification_options==NOTIFICATION_HOST_ALL && notification_type==HOST_NOTIFICATION)
					show_entry=TRUE;
				else if(notification_options==NOTIFICATION_SERVICE_ALL && notification_type==SERVICE_NOTIFICATION)
					show_entry=TRUE;
				else if(notification_detail_type & notification_options)
					show_entry=TRUE;
				else 
					show_entry=FALSE;
			        }

			/* make sure user has authorization to view this notification */
			if(notification_type==HOST_NOTIFICATION){
				temp_host=find_host(host_name);
				if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
					show_entry=FALSE;
				else
					total_notifications++;
			        }
			else{
				temp_service=find_service(host_name,service_name);
				if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
					show_entry=FALSE;
				else
					total_notifications++;

			        }

			if(show_entry==TRUE){
				if(odd){
					odd=0;
					printf("<tr bgcolor=%s>\n",html_table_bg_odd_color);
				        }
				else{
					odd=1;
					printf("<tr bgcolor=%s>\n",html_table_bg_even_color);
				        }
				printf("<td><a href=\"%s?type=hosts#%s\"><font size=-1>%s</font></a></td>\n",CONFIG_CGI,url_encode(host_name),host_name);
				if(notification_type==SERVICE_NOTIFICATION){
					printf("<td><a href=\"%s?type=services#%s;",CONFIG_CGI,url_encode(host_name));
					printf("%s\"><font size=-1>%s</font></a></td>\n",url_encode(service_name),service_name);
				        }
				else
					printf("<td><font size=-1>N/A</font></td>\n");
				printf("<td bgcolor=\"%s\"><a href=\"%sstatuslevels.html#%s\"><font size=-1>%s</font></a></td>\n",alert_color,url_docs_path,alert_level,alert_level);
				printf("<td><font size=-1>%s</font></td>\n",date_time);
				printf("<td><a href=\"%s?type=contacts#%s\"><font size=-1>%s</font></a></td>\n",CONFIG_CGI,url_encode(contact_name),contact_name);
				printf("<td><a href=\"%s?type=commands#%s\"><font size=-1>%s</font></a></td>\n",CONFIG_CGI,url_encode(method_name),method_name);
				printf("<td><font size=-1>%s</font></td>\n",temp_buffer);
				printf("</tr>\n");
			        }
		        }
	        }


	printf("</table>\n");
	printf("</div>\n");
	printf("</p>\n");

	if(total_notifications==0){
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("No %s notifications have been recorded in %s log file",(find_type==FIND_CONTACT)?"contact":"host",(log_archive==0)?"the current":"this archived");
		printf("</strong></font></div></p>\n");
	        }

	if(use_lifo==TRUE)
		free_lifo_memory();
	else
		fclose(fp);

	return;
        }
