/**************************************************************************
 *
 * STATUS.C -  NetSaint Status CGI
 *
 * Copyright (c) 1999-2000 Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 01-17-2000
 *
 * License:
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include "../common/common.h"

#include "cgiutils.h"
#include "getcgi.h"
#include "auth.h"

extern char            netsaint_process_info[MAX_INPUT_BUFFER];
extern int             netsaint_process_state;
extern int             refresh_rate;

#define MAX_MESSAGE_BUFFER		4096

#define DISPLAY_HOSTS			0
#define DISPLAY_HOSTGROUPS		1

#define STYLE_OVERVIEW			0
#define STYLE_DETAIL			1

void show_host_status_totals(void);
void show_service_status_totals(void);
void show_service_detail(void);
void show_hostgroup_overviews(void);
void show_hostgroup_overview(hostgroup *);
void show_hostgroup_member_overview(hoststatus *,int);
void show_hostgroup_member_service_status_totals(char *);

void grab_host_alerts(void);
void alert_window(char *);

void document_header(void);
void document_footer(void);
int process_cgivars(void);

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_html_path[MAX_FILENAME_LENGTH];
extern char url_docs_path[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char url_logo_images_path[MAX_FILENAME_LENGTH];
extern char url_cgibin_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];
extern char status_file[MAX_FILENAME_LENGTH];
extern char comment_file[MAX_FILENAME_LENGTH];

extern char html_table_bg_color[MAX_COLOR_LENGTH];
extern char html_table_fg_color[MAX_COLOR_LENGTH];
extern char html_table_bg_odd_color[MAX_COLOR_LENGTH];
extern char html_table_bg_even_color[MAX_COLOR_LENGTH];

extern int suppress_alert_window;

extern hostgroup *hostgroup_list;
extern hoststatus *hoststatus_list;
extern servicestatus *servicestatus_list;

authdata current_authdata;

char alert_message[MAX_MESSAGE_BUFFER];
char host_name[MAX_HOSTNAME_LENGTH];
char hostgroup_name[MAX_HOSTGROUPNAME_LENGTH];
int host_alert=FALSE;
int show_all_hosts=FALSE;
int show_all_hostgroups=FALSE;
int display_type=DISPLAY_HOSTS;
int overview_columns=3;
int display_popup=TRUE;
int hostgroup_style_type=STYLE_OVERVIEW;
int service_status_types=SERVICE_PENDING|SERVICE_RECOVERY|SERVICE_OK|SERVICE_UNKNOWN|SERVICE_WARNING|SERVICE_CRITICAL|SERVICE_HOST_DOWN|SERVICE_UNREACHABLE;
int host_status_types=HOST_PENDING|HOST_UP|HOST_DOWN|HOST_UNREACHABLE;
int all_service_status_types=SERVICE_PENDING|SERVICE_RECOVERY|SERVICE_OK|SERVICE_UNKNOWN|SERVICE_WARNING|SERVICE_CRITICAL|SERVICE_HOST_DOWN|SERVICE_UNREACHABLE;
int all_host_status_types=HOST_PENDING|HOST_UP|HOST_DOWN|HOST_UNREACHABLE;


int main(void){
	int result=OK;
	int error=FALSE;
	time_t current_time;
	char date_time[48];
	
	error=process_cgivars();

	if(error==TRUE){
		document_header();
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied");
		printf("</strong></font></div></p>\n");
		document_footer();
	        }

	else{
	
		reset_cgi_vars();

		result=read_cgi_config_file(DEFAULT_CGI_CONFIG_FILE);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file '%s' for reading!\n",DEFAULT_CGI_CONFIG_FILE);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file '%s' for reading!\n",main_config_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_status_log(status_file);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open status log '%s' for reading!\n",status_file);
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_all_host_config_files(READ_HOSTGROUPS|READ_CONTACTGROUPS|READ_HOSTS|READ_SERVICES);
		if(result==ERROR){
			document_header();
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open host configuration file(s) for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
                        }

		/* read in all host and service comments */
		read_comment_file(comment_file);
	        }


	document_header();

	/* begin top table */
	printf("<p>\n");
	printf("<table border=0 width=100%% cellspacing=0 cellpadding=0>\n");
	printf("<tr>\n");
	printf("<td align=left valign=top>\n");

	/* left column of the first row */
	printf("<table border=1>\n");
	printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
	printf("<font size=+2><b>Current Network Status</b></font><br>\n");
	printf("<font size=-2>\n");
	time(&current_time);
	get_time_string(&current_time,date_time,(int)sizeof(date_time));
	printf("Last Updated: %s<BR>",date_time);
	printf("Updated every %d seconds<br>\n",refresh_rate);
	printf("NetSaint Network Monitor - <a href=\"http://www.netsaint.org\" target=\"_new\">www.netsaint.org</a><br>\n");
	get_netsaint_process_info();
	printf("<font color=\"%s\">%s</font><br>\n",(netsaint_process_state==STATE_OK)?"":CRITICAL_COLOR,(netsaint_process_state==STATE_OK)?"NetSaint process is running...":"Warning: NetSaint process may not be running!");
	printf("</font>\n");
	printf("</td></tr>\n");

	if(display_type==DISPLAY_HOSTS){

		printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
		printf("<font size=-2>\n");
		printf("<a href=\"%s?host=%s\">View history for %s</a><br>\n",HISTORY_CGI,(show_all_hosts==TRUE)?"all":url_encode(host_name),(show_all_hosts==TRUE)?"all hosts":"this host");
		printf("<a href=\"%s?host=%s\">View notifications for %s</a>\n",NOTIFICATIONS_CGI,(show_all_hosts==TRUE)?"all":url_encode(host_name),(show_all_hosts==TRUE)?"all hosts":"this host");
		printf("</font>\n");
		printf("</td></tr>\n");

		if(show_all_hosts==FALSE){
			printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
			printf("<font size=-2>\n");
			printf("<a href=\"%s?host=all%s\">View status detail for all hosts</a>\n",STATUS_CGI,(display_popup==TRUE)?"":"&nopopup");
			printf("</font>\n");
			printf("</td></tr>\n");
	                }

	        }

	else{

		if(show_all_hostgroups==FALSE){

			if(hostgroup_style_type==STYLE_OVERVIEW){
				printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
				printf("<font size=-2>\n");
				printf("<a href=\"%s?hostgroup=all%s\">View status overview for all host groups</a>\n",STATUS_CGI,(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
				printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
				printf("<font size=-2>\n");
				printf("<a href=\"%s?hostgroup=%s&style=detail%s\">View status detail for this host group</a>\n",STATUS_CGI,url_encode(hostgroup_name),(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
			else{
				printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
				printf("<font size=-2>\n");
				printf("<a href=\"%s?hostgroup=%s%s\">View status overview for this host group</a>\n",STATUS_CGI,url_encode(hostgroup_name),(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
				printf("<tr><td bgcolor=%s>\n",html_table_bg_odd_color);
				printf("<font size=-2>\n");
				printf("<a href=\"%s?hostgroup=all&style=detail%s\">View status detail for all host groups</a>\n",STATUS_CGI,(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
		        }
		else{

			if(hostgroup_style_type==STYLE_OVERVIEW){
				printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
				printf("<font size=-2>\n");
				printf("<a href=\"%s?hostgroup=all&style=detail%s\">View status detail for all host groups</a>\n",STATUS_CGI,(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
			else{
				printf("<tr><td bgcolor=%s>\n",html_table_bg_even_color);
				printf("<font size=-2>\n");
				printf("<a href=\"%s?hostgroup=all%s\">View status overview for all host groups</a>\n",STATUS_CGI,(display_popup==TRUE)?"":"&nopopup");
				printf("</font>\n");
				printf("</td></tr>\n");
			        }
		        }
	        }

	printf("</table>\n");
	printf("</td>\n");

	/* get authentication information */
	get_authentication_information(&current_authdata);

	/* middle column of top row */
	printf("<td align=center valign=bottom>\n");
	show_host_status_totals();
	printf("</td>\n");


	/* right hand column of top row */
	printf("<td align=right valign=bottom>\n");
	show_service_status_totals();
	printf("</td>\n");

	/* end of top table */
	printf("</tr>\n");
	printf("</table>\n");
	printf("</p>\n");

	/* bottom portion of screen - service or hostgroup detail */
	if(display_type==DISPLAY_HOSTS)
		show_service_detail();
	else{
		if(hostgroup_style_type==STYLE_OVERVIEW)
			show_hostgroup_overviews();
		else
			show_service_detail();
	        }

	/* show the JavaScript alert window if a host is down or unreachable */
	if(suppress_alert_window==FALSE){
		grab_host_alerts();
		if(host_alert==TRUE)
			alert_window(alert_message);
	        }

	document_footer();

	free_memory();

	return OK;
        }


void document_header(void){
	char date_time[48];
	time_t current_time;
	time_t expire_time;


	printf("Cache-Control: no-store\n");
	printf("Pragma: no-cache\n");
	printf("Refresh: %d\n",refresh_rate);

	time(&current_time);
	get_expire_time_string(&current_time,date_time,(int)sizeof(date_time));
	printf("Last-Modified: %s\n",date_time);

	expire_time=(time_t)0L;
	get_expire_time_string(&expire_time,date_time,(int)sizeof(date_time));
	printf("Expires: %s\n",date_time);

	printf("Content-type: text/html\n\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<title>\n");
	printf("Current Service Status\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body onunload=\"if (window.wnd && wnd.close) wnd.close(); return true;\" bgcolor=\"%s\">\n",HTML_BACKGROUND_COLOR);

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the hostgroup argument */
		else if(!strcmp(variables[x],"hostgroup")){
			display_type=DISPLAY_HOSTGROUPS;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTGROUPNAME_LENGTH-1){
				strcpy(hostgroup_name,"all");
			        }
			else{
				strncpy(hostgroup_name,variables[x],sizeof(hostgroup_name));
				hostgroup_name[sizeof(hostgroup_name)-1]='\x0';
			        }

			if(!strcmp(hostgroup_name,"all"))
				show_all_hostgroups=TRUE;
			else
				show_all_hostgroups=FALSE;
		        }

		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			display_type=DISPLAY_HOSTS;
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1){
				strcpy(host_name,"all");
			        }
			else{
				strncpy(host_name,variables[x],sizeof(host_name));
				host_name[sizeof(host_name)-1]='\x0';
			        }

			if(!strcmp(host_name,"all"))
				show_all_hosts=TRUE;
			else
				show_all_hosts=FALSE;
		        }

		/* we found the columns argument */
		else if(!strcmp(variables[x],"columns")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			overview_columns=atoi(variables[x]);
			if(overview_columns<=0)
				overview_columns=1;
		        }

		/* we found the service status type argument */
		else if(!strcmp(variables[x],"servicestatustypes")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			service_status_types=atoi(variables[x]);
		        }

		/* we found the host status type argument */
		else if(!strcmp(variables[x],"hoststatustypes")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			host_status_types=atoi(variables[x]);
		        }

		/* we found the popup window argument */
		else if(!strcmp(variables[x],"nopopup"))
			display_popup=FALSE;

		/* we found the hostgroup style argument */
		else if(!strcmp(variables[x],"style")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			if(!strcmp(variables[x],"detail"))
				hostgroup_style_type=STYLE_DETAIL;
		        }
	        }

	return error;
        }



/* grab alert messages for all hosts that are down or unreachable */
void grab_host_alerts(void){
	char message_buffer[MAX_INPUT_BUFFER];
	hoststatus *temp_hoststatus=NULL;
	host *temp_host;

	/* check the status of all hosts... */
	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){

		/* make sure the user is authorized to see this host... */
		temp_host=find_host(temp_hoststatus->host_name);
		if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
			continue;

		/* if the host is down or unreachable and user is authorized to view this host, add an alert message... */
		if(temp_hoststatus->status==HOST_DOWN || temp_hoststatus->status==HOST_UNREACHABLE){

			if(host_alert==FALSE){
				host_alert=TRUE;
				strcpy(alert_message,"");
			        }

			/* get the alert message to print */
			snprintf(message_buffer,sizeof(message_buffer),"<tr><td valign='center'><IMG SRC='%s%s' BORDER=0 ALIGN='CENTER'></td><td>Host <b>%s</b> is <b>%s</b>!</td></tr>",url_images_path,HOST_CRITICAL_ICON,temp_hoststatus->host_name,(temp_hoststatus->status==HOST_DOWN)?"down":"unreachable");
			message_buffer[sizeof(message_buffer)-1]='\x0';

			/* add the current alert on the end of the alert message buffer */
			strncat(alert_message,message_buffer,(int)(sizeof(alert_message)-strlen(alert_message)-1));
			alert_message[sizeof(alert_message)-1]='\x0';
		        }
	        }

	return;
        }


/* display the popup window if there are hosts that are down or unreachable */
void alert_window(char *alert_message){

	if(display_popup==TRUE){

		printf("<script language=\"javascript\">\n");
		printf("<!-- Hide code from old browsers\n");

		printf("var wnd;\n");
		printf("wnd = window.open(\"\",\"alert\",\"width=400,height=200,scrollbars=yes,statusbar=no,toolbar=no\");\n");
		printf("wnd.document.write(\"<title>NetSaint Host Alert Console</title>\");\n");
		printf("wnd.document.write(\"<body bgcolor='%s' text='#000000'>\");\n",html_table_bg_odd_color);

		printf("wnd.document.write(\"<p>\");\n");
		printf("wnd.document.write(\"<center><font size=-2>");
		printf("<a href='' onclick='self.close()'>Close This Window</a>");
		printf("&nbsp;&nbsp;|&nbsp;&nbsp;");
		printf("<a href='");
		printf("%s?",STATUS_CGI);

		if(display_type==DISPLAY_HOSTS){
			printf("host=%s",host_name);
		        }
		else{
			printf("hostgroup=%s",hostgroup_name);
			if(hostgroup_style_type==STYLE_DETAIL)
				printf("&style=detail");
			else
				printf("&columns=%d",overview_columns);
		        }

		printf("&nopopup");
		printf("' target='main'>Disable This Window</a>");
		printf("</font></center>\");\n");
		printf("wnd.document.write(\"</p>\");\n");
		printf("wnd.document.write(\"<p>\");\n");
		printf("wnd.document.write(\"<table border='0'>\");\n");
		printf("wnd.document.write(\"%s\");\n",alert_message);
		printf("wnd.document.write(\"</table>\");\n");
		printf("wnd.document.write(\"</p>\");\n");
		printf("wnd.document.write(\"</body>\");\n");
		printf("//-->\n");
		printf("</script>\n");
	        }

	return;
        }



/* display table with service status totals... */
void show_service_status_totals(void){
	int total_ok=0;
	int total_warning=0;
	int total_unknown=0;
	int total_critical=0;
	int total_pending=0;
	int total_services=0;
	int total_problems=0;
	servicestatus *temp_servicestatus;
	service *temp_service;
	host *temp_host;
	int count_service;


	/* check the status of all services... */
	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){

		/* find the host and service... */
		temp_host=find_host(temp_servicestatus->host_name);
		temp_service=find_service(temp_servicestatus->host_name,temp_servicestatus->description);

		/* make sure user has rights to see this service... */
		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
			continue;

		count_service=0;

		if(display_type==DISPLAY_HOSTS && (show_all_hosts==TRUE || !strcmp(host_name,temp_servicestatus->host_name)))
			count_service=1;
		else if(display_type==DISPLAY_HOSTGROUPS && (show_all_hostgroups==TRUE || is_member_of_hostgroup(find_hostgroup(hostgroup_name),temp_host)))
			count_service=1;

		if(count_service){

			if(temp_servicestatus->status==SERVICE_CRITICAL)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_HOST_DOWN)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_UNREACHABLE)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_WARNING)
				total_warning++;
			else if(temp_servicestatus->status==SERVICE_UNKNOWN)
				total_unknown++;
			else if(temp_servicestatus->status==SERVICE_RECOVERY)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_OK)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_PENDING)
				total_pending++;
			else
				total_ok++;
		        }
	        }

	total_services=total_ok+total_unknown+total_warning+total_critical+total_pending;
	total_problems=total_unknown+total_warning+total_critical;


	printf("<DIV ALIGN=\"CENTER\">\n");
	printf("<B>Service Status Totals</B>\n");

	printf("<TABLE BORDER=0>\n");
	printf("<TR><TD>\n");


	printf("<TABLE BORDER=\"1\" bgcolor=\"#EEEEEE\">\n");
	printf("<TR>\n");

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else
		printf("hostgroup=%s&style=detail",hostgroup_name);
	if(display_popup==FALSE)
		printf("&nopopup");
	printf("&servicestatustypes=%d",SERVICE_OK|SERVICE_RECOVERY);
	printf("&hoststatustypes=%d'>",host_status_types);
	printf("<FONT color=\"%s\">Ok</FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else
		printf("hostgroup=%s&style=detail",hostgroup_name);
	if(display_popup==FALSE)
		printf("&nopopup");
	printf("&servicestatustypes=%d",SERVICE_WARNING);
	printf("&hoststatustypes=%d'>",host_status_types);
	printf("<FONT color=\"%s\">Warning</FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else
		printf("hostgroup=%s&style=detail",hostgroup_name);
	if(display_popup==FALSE)
		printf("&nopopup");
	printf("&servicestatustypes=%d",SERVICE_UNKNOWN);
	printf("&hoststatustypes=%d'>",host_status_types);
	printf("<FONT color=\"%s\">Unknown</FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else
		printf("hostgroup=%s&style=detail",hostgroup_name);
	if(display_popup==FALSE)
		printf("&nopopup");
	printf("&servicestatustypes=%d",SERVICE_CRITICAL|SERVICE_HOST_DOWN|SERVICE_UNREACHABLE);
	printf("&hoststatustypes=%d'>",host_status_types);
	printf("<FONT color=\"%s\">Critical</FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else
		printf("hostgroup=%s&style=detail",hostgroup_name);
	if(display_popup==FALSE)
		printf("&nopopup");
	printf("&servicestatustypes=%d",SERVICE_PENDING);
	printf("&hoststatustypes=%d'>",host_status_types);
	printf("<FONT color=\"%s\">Pending</FONT></A></TH>\n",html_table_fg_color);

	printf("</TR>\n");

	printf("<TR>\n");


	/* total services ok */
        printf("<TD align=\"center\"");
	if(total_ok>0)
		printf(" bgcolor=\"%s\"",OK_COLOR);
	printf(">%d",total_ok);
	printf("</TD>\n");

	/* total services in warning state */
        printf("<TD align=\"center\"");
	if(total_warning>0)
		printf(" bgcolor=\"%s\"",WARNING_COLOR);
	printf(">%s%d%s",(total_warning>0)?"<BLINK><B>":"",total_warning,(total_warning>0)?"</B></BLINK>":"");
	printf("</TD>\n");

	/* total services in unknown state */
        printf("<TD align=\"center\"");
	if(total_unknown>0)
		printf(" bgcolor=\"%s\"",UNKNOWN_COLOR);
	printf(">%s%d%s",(total_unknown>0)?"<BLINK><B>":"",total_unknown,(total_unknown>0)?"</B></BLINK>":"");
	printf("</TD>\n");

	/* total services in critical state */
        printf("<TD align=\"center\"");
	if(total_critical>0)
		printf(" bgcolor=\"%s\"",CRITICAL_COLOR);
	printf(">%s%d%s",(total_critical>0)?"<BLINK><B>":"",total_critical,(total_critical>0)?"</B></BLINK>":"");
	printf("</TD>\n");

	/* total services in pending state */
        printf("<TD align=\"center\"");
	if(total_pending>0)
		printf(" bgcolor=\"%s\"",PENDING_COLOR);
	printf(">%d",total_pending);
	printf("</TD>\n");

	printf("</TR>\n");
	printf("</TABLE>\n");

	printf("</TD></TR><TR><TD ALIGN=CENTER>\n");

	printf("<TABLE BORDER=\"1\" bgcolor=\"#EEEEEE\">\n");
	printf("<TR>\n");

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else
		printf("hostgroup=%s&style=detail",hostgroup_name);
	if(display_popup==FALSE)
		printf("&nopopup");
	printf("&servicestatustypes=%d",SERVICE_UNKNOWN|SERVICE_WARNING|SERVICE_CRITICAL|SERVICE_HOST_DOWN|SERVICE_UNREACHABLE);
	printf("&hoststatustypes=%d'>",host_status_types);
	printf("<FONT color=\"%s\"><I>All Problems</I></FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else
		printf("hostgroup=%s&style=detail",hostgroup_name);
	if(display_popup==FALSE)
		printf("&nopopup");
	printf("&hoststatustypes=%d'>",host_status_types);
	printf("<FONT color=\"%s\"><I>All Types</I></FONT></A></TH>\n",html_table_fg_color);


	printf("</TR><TR>\n");

	/* total service problems */
        printf("<TD align=\"center\">");
	printf("<I>%s%d%s</I>",(total_problems>0)?"<BLINK><B>":"",total_problems,(total_problems>0)?"</B></BLINK>":"");
	printf("</TD>\n");

	/* total services */
        printf("<TD align=\"center\"><I>%d</I></TD>",total_services);

	printf("</TR>\n");
	printf("</TABLE>\n");

	printf("</TD></TR>\n");
	printf("</TABLE>\n");

	printf("</DIV>\n");

	return;
        }


/* display a table with host status totals... */
void show_host_status_totals(void){
	int total_up=0;
	int total_down=0;
	int total_unreachable=0;
	int total_pending=0;
	int total_hosts=0;
	int total_problems=0;
	hoststatus *temp_hoststatus;
	host *temp_host;
	int count_host;


	/* check the status of all hosts... */
	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){

		/* find the host... */
		temp_host=find_host(temp_hoststatus->host_name);

		/* make sure user has rights to view this host */
		if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
			continue;

		count_host=0;

		if(display_type==DISPLAY_HOSTS && (show_all_hosts==TRUE || !strcmp(host_name,temp_hoststatus->host_name)))
			count_host=1;
		else if(display_type==DISPLAY_HOSTGROUPS && (show_all_hostgroups==TRUE || is_member_of_hostgroup(find_hostgroup(hostgroup_name),temp_host)))
			count_host=1;

		if(count_host){

			if(temp_hoststatus->status==HOST_UP)
				total_up++;
			else if(temp_hoststatus->status==HOST_DOWN)
				total_down++;
			else if(temp_hoststatus->status==HOST_UNREACHABLE)
				total_unreachable++;
			else if(temp_hoststatus->status==HOST_PENDING)
				total_pending++;
			else
				total_up++;
		        }
	        }

	total_hosts=total_up+total_down+total_unreachable+total_pending;
	total_problems=total_down+total_unreachable;

	printf("<DIV ALIGN=\"CENTER\">\n");
	if(display_type==DISPLAY_HOSTGROUPS || (display_type==DISPLAY_HOSTS && show_all_hosts==TRUE))
		printf("<B>Host Status Totals</B>\n");
	else
		printf("<B>Host '%s' Status</B>\n",host_name);

	printf("<TABLE BORDER=0>\n");
	printf("<TR><TD>\n");


	printf("<TABLE BORDER=1 bgcolor=\"#EEEEEE\">\n");
	printf("<TR>\n");

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else{
		printf("hostgroup=%s",hostgroup_name);
		if((service_status_types!=all_service_status_types) || hostgroup_style_type==STYLE_DETAIL)
			printf("&style=detail");
	        }
	if(display_popup==FALSE)
		printf("&nopopup");
	if(service_status_types!=all_service_status_types)
		printf("&servicestatustypes=%d",service_status_types);
	printf("&hoststatustypes=%d'>",HOST_UP);
	printf("<FONT color=\"%s\">Up</FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else{
		printf("hostgroup=%s",hostgroup_name);
		if((service_status_types!=all_service_status_types) || hostgroup_style_type==STYLE_DETAIL)
			printf("&style=detail");
	        }
	if(display_popup==FALSE)
		printf("&nopopup");
	if(service_status_types!=all_service_status_types)
		printf("&servicestatustypes=%d",service_status_types);
	printf("&hoststatustypes=%d'>",HOST_DOWN);
	printf("<FONT color=\"%s\">Down</FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else{
		printf("hostgroup=%s",hostgroup_name);
		if((service_status_types!=all_service_status_types) || hostgroup_style_type==STYLE_DETAIL)
			printf("&style=detail");
	        }
	if(display_popup==FALSE)
		printf("&nopopup");
	if(service_status_types!=all_service_status_types)
		printf("&servicestatustypes=%d",service_status_types);
	printf("&hoststatustypes=%d'>",HOST_UNREACHABLE);
	printf("<FONT color=\"%s\">Unreachable</FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else{
		printf("hostgroup=%s",hostgroup_name);
		if((service_status_types!=all_service_status_types) || hostgroup_style_type==STYLE_DETAIL)
			printf("&style=detail");
	        }
	if(display_popup==FALSE)
		printf("&nopopup");
	if(service_status_types!=all_service_status_types)
		printf("&servicestatustypes=%d",service_status_types);
	printf("&hoststatustypes=%d'>",HOST_PENDING);
	printf("<FONT color=\"%s\">Pending</FONT></A></TH>\n",html_table_fg_color);

	printf("</TR>\n");


	printf("<TR>\n");

	/* total hosts up */
        printf("<TD align=\"center\"");
	if(total_up>0)
		printf(" bgcolor=\"%s\"",HOST_UP_COLOR);
	printf(">%d",total_up);
	printf("</TD>\n");

	/* total hosts down */
        printf("<TD align=\"center\"");
	if(total_down>0)
		printf(" bgcolor=\"%s\"",HOST_DOWN_COLOR);
	printf(">%s%d%s",(total_down>0)?"<BLINK><B>":"",total_down,(total_down>0)?"</B></BLINK>":"");
	printf("</TD>\n");

	/* total hosts up */
        printf("<TD align=\"center\"");
	if(total_unreachable>0)
		printf(" bgcolor=\"%s\"",HOST_UNREACHABLE_COLOR);
	printf(">%s%d%s",(total_unreachable>0)?"<BLINK><B>":"",total_unreachable,(total_unreachable>0)?"</B></BLINK>":"");
	printf("</TD>\n");

	/* total hosts pending */
        printf("<TD align=\"center\"");
	if(total_pending>0)
		printf(" bgcolor=\"%s\"",HOST_PENDING_COLOR);
	printf(">%d",total_pending);
	printf("</TD>\n");

	printf("</TR>\n");
	printf("</TABLE>\n");

	printf("</TD></TR><TR><TD ALIGN=CENTER>\n");

	printf("<TABLE BORDER=\"1\" bgcolor=\"#EEEEEE\">\n");
	printf("<TR>\n");

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else{
		printf("hostgroup=%s",hostgroup_name);
		if((service_status_types!=all_service_status_types) || hostgroup_style_type==STYLE_DETAIL)
			printf("&style=detail");
	        }
	if(display_popup==FALSE)
		printf("&nopopup");
	if(service_status_types!=all_service_status_types)
		printf("&servicestatustypes=%d",service_status_types);
	printf("&hoststatustypes=%d'>",HOST_DOWN|HOST_UNREACHABLE);
	printf("<FONT color=\"%s\"><I>All Problems</I></FONT></A></TH>\n",html_table_fg_color);

	printf("<TH align=left bgcolor=\"%s\">",html_table_bg_color);
	printf("<a href='%s?",STATUS_CGI);
	if(display_type==DISPLAY_HOSTS)
		printf("host=%s",host_name);
	else{
		printf("hostgroup=%s",hostgroup_name);
		if((service_status_types!=all_service_status_types) || hostgroup_style_type==STYLE_DETAIL)
			printf("&style=detail");
	        }
	if(display_popup==FALSE)
		printf("&nopopup");
	if(service_status_types!=all_service_status_types)
		printf("&servicestatustypes=%d",service_status_types);
	printf("'>");
	printf("<FONT color=\"%s\"><I>All Types</I></FONT></A></TH>\n",html_table_fg_color);

	printf("</TR><TR>\n");

	/* total hosts with problems */
        printf("<TD align=\"center\">");
	printf("<I>%s%d%s</I>",(total_problems>0)?"<BLINK><B>":"",total_problems,(total_problems>0)?"</B></BLINK>":"");
	printf("</TD>\n");

	/* total hosts */
        printf("<TD align=\"center\"><I>%d</I></TD>",total_hosts);

	printf("</TR>\n");
	printf("</TABLE>\n");

	printf("</TD></TR>\n");
	printf("</TABLE>\n");

	printf("</DIV>\n");

	return;
        }



/* display a detailed listing of the status of all services... */
void show_service_detail(void){
	time_t t;
	char date_time[48];
	char status[MAX_INPUT_BUFFER];
	char status_color[MAX_INPUT_BUFFER];
	char status_bg_color[MAX_INPUT_BUFFER];
	char last_host[MAX_INPUT_BUFFER]="";
	int new_host=FALSE;
	servicestatus *temp_status;
	hostgroup *temp_hostgroup;
	hostextinfo *temp_hostextinfo;
	hoststatus *temp_hoststatus;
	host *temp_host;
	service *temp_service;
	int odd=0;
	int total_comments=0;
	int user_has_seen_something=FALSE;
	int found=0;


	printf("<DIV ALIGN=LEFT>\n");
	printf("<FONT SIZE=-2>\n");

	if(host_status_types!=all_host_status_types){
		found=0;
		printf("Displaying services with host status level(s) of:<b>");
		if(host_status_types & HOST_PENDING){
			printf(" PENDING");
			found=1;
		        }
		if(host_status_types & HOST_UP){
			printf("%s UP",(found==1)?",":"");
			found=1;
		        }
		if(host_status_types & HOST_DOWN){
			printf("%s DOWN",(found==1)?",":"");
			found=1;
		        }
		if(host_status_types & HOST_UNREACHABLE)
			printf("%s UNREACHABLE",(found==1)?",":"");
		printf("</b> \n");
	        }
	else
		printf("Displaying services with <b>all</b> host status levels \n");

	if(service_status_types!=all_service_status_types){
		printf("<i>and</i> service status level(s) of:<b>");
		if(service_status_types & SERVICE_PENDING){
			printf(" PENDING");
			found=1;
		        }
		if(service_status_types & SERVICE_RECOVERY){
			printf("%s RECOVERY",(found==1)?",":"");
			found=1;
		        }
		if(service_status_types & SERVICE_OK){
			printf("%s OK",(found==1)?",":"");
			found=1;
		        }
		if(service_status_types & SERVICE_UNKNOWN){
			printf("%s UNKNOWN",(found==1)?",":"");
			found=1;
		        }
		if(service_status_types & SERVICE_WARNING){
			printf("%s WARNING",(found==1)?",":"");
			found=1;
		        }
		if(service_status_types & SERVICE_CRITICAL){
			printf("%s CRITICAL",(found==1)?",":"");
			found=1;
		        }
		if(service_status_types & SERVICE_HOST_DOWN){
			printf("%s HOST_DOWN",(found==1)?",":"");
			found=1;
		        }
		if(service_status_types & SERVICE_UNREACHABLE)
			printf("%s UNREACHABLE",(found==1)?",":"");
		printf("</b><br>\n");
	        }
	else
		printf("<i>and</i> <b>all</b> service status levels...<br>\n");

	printf("</FONT>\n");
	printf("</DIV>\n");

	printf("<P>\n");
	printf("<DIV ALIGN=CENTER>\n");
	if(display_type==DISPLAY_HOSTS){
		if(show_all_hosts==TRUE)
			printf("<B>Service Status Details For All Hosts</B>\n");
		else
			printf("<B>Service Status Details For Host '%s'</B>\n",host_name);
	        }
	else{
		if(show_all_hostgroups==TRUE)
			printf("<B>Service Status Details For All Host Groups</B>\n");
		else
			printf("<B>Service Status Details For Host Group '%s'</B>\n",hostgroup_name);
	        }
	printf("<TABLE BORDER=0 width=100%%>\n");
	printf("<TR>\n");
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Host</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Service</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Status</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Last Updated</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Attempt</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("<TH align=left bgcolor=\"%s\"><FONT COLOR=\"%s\">Service Information</FONT></TH>\n",html_table_bg_color,html_table_fg_color);
	printf("</TR>\n");


	/* check all services... */
	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next){

		/* get host and service */
		temp_host=find_host(temp_status->host_name);
		temp_service=find_service(temp_status->host_name,temp_status->description);

		/* make sure user has rights to see this... */
		if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
			continue;

		user_has_seen_something=TRUE;

		/* get the host status information */
		temp_hoststatus=find_hoststatus(temp_status->host_name);

		/* see if we should display services for hosts with tis type of status */
		if(!(host_status_types & temp_hoststatus->status))
			continue;

		/* see if we should display this type of service status */
		if(!(service_status_types & temp_status->status))
			continue;									   

		if(display_type==DISPLAY_HOSTGROUPS){

			/* see if this host is a member of the hostgroup */
			if(show_all_hostgroups==FALSE){
				temp_hostgroup=find_hostgroup(hostgroup_name);
				if(temp_hostgroup==NULL)
					continue;
				if(is_member_of_hostgroup(temp_hostgroup,temp_host)==FALSE)
					continue;
		                }
		        }

		/* get the last service check time */
		t=temp_status->last_check;
		get_time_string(&t,date_time,(int)sizeof(date_time));
		if((unsigned long)temp_status->last_check==0L)
			strcpy(date_time,"N/A");


		if(temp_status->status==SERVICE_PENDING){
			strncpy(status,"PENDING",sizeof(status));
			strncpy(status_color,PENDING_COLOR,sizeof(status_color));
		        }
		else if(temp_status->status==SERVICE_OK){
			strncpy(status,"OK",sizeof(status));
			strncpy(status_color,OK_COLOR,sizeof(status_color));
		        }
		else if(temp_status->status==SERVICE_RECOVERY){
			strncpy(status,"RECOVERY",sizeof(status));
			strncpy(status_color,RECOVERY_COLOR,sizeof(status_color));
		        }
		else if(temp_status->status==SERVICE_WARNING){
			strncpy(status,"WARNING",sizeof(status));
			strncpy(status_color,WARNING_COLOR,sizeof(status_color));
			strncpy(status_bg_color,WARNING_BG_COLOR,sizeof(status_bg_color));
		        }
		else if(temp_status->status==SERVICE_UNKNOWN){
			strncpy(status,"UNKNOWN",sizeof(status));
			strncpy(status_color,UNKNOWN_COLOR,sizeof(status_color));
			strncpy(status_bg_color,UNKNOWN_BG_COLOR,sizeof(status_bg_color));
		        }
		else if(temp_status->status==SERVICE_CRITICAL){
			strncpy(status,"CRITICAL",sizeof(status));
			strncpy(status_color,CRITICAL_COLOR,sizeof(status_color));
			strncpy(status_bg_color,CRITICAL_BG_COLOR,sizeof(status_bg_color));
		        }
		else if(temp_status->status==SERVICE_HOST_DOWN){
			strncpy(status,"HOST DOWN",sizeof(status));
			strncpy(status_color,CRITICAL_COLOR,sizeof(status_color));
			strncpy(status_bg_color,CRITICAL_BG_COLOR,sizeof(status_bg_color));
		        }
		else if(temp_status->status==SERVICE_UNREACHABLE){
			strncpy(status,"UNREACHABLE",sizeof(status));
			strncpy(status_color,CRITICAL_COLOR,sizeof(status_color));
			strncpy(status_bg_color,CRITICAL_BG_COLOR,sizeof(status_bg_color));
		        }

		status[sizeof(status)-1]='\x0';
		status_color[sizeof(status_color)-1]='\x0';
		status_bg_color[sizeof(status_bg_color)-1]='\x0';


		if((display_type==DISPLAY_HOSTS && (show_all_hosts==TRUE || !strcmp(host_name,temp_status->host_name))) || display_type==DISPLAY_HOSTGROUPS ){

			if(strcmp(last_host,temp_status->host_name))
				new_host=TRUE;
			else
				new_host=FALSE;

			if(new_host==TRUE){
				if(strcmp(last_host,"")){
					printf("<TR><TD colspan=6><FONT SIZE=-2></FONT></TD></TR>\n");
					printf("<TR><TD colspan=6><FONT SIZE=-2></FONT></TD></TR>\n");
				        }
				else{
				        }
			        }

			if(odd)
				odd=0;
			else
				odd=1;

			if(temp_status->status==SERVICE_OK || temp_status->status==SERVICE_RECOVERY || temp_status->status==SERVICE_PENDING)
				printf("<TR bgcolor=%s>\n",(odd)?html_table_bg_odd_color:html_table_bg_even_color);
			else
				printf("<TR bgcolor=%s>\n",status_bg_color);

			if(new_host==TRUE){

				/* find extended information for this host */
				temp_hostextinfo=find_hostextinfo(temp_status->host_name);

				if(temp_hoststatus->status==HOST_UP || temp_hoststatus->status==HOST_PENDING)
					strncpy(status_bg_color,(odd)?html_table_bg_odd_color:html_table_bg_even_color,sizeof(status_bg_color));
				else
					strncpy(status_bg_color,CRITICAL_BG_COLOR,sizeof(status_bg_color));
				status_bg_color[sizeof(status_bg_color)-1]='\x0';

				printf("<TD bgcolor=%s>",status_bg_color);

				printf("<TABLE BORDER=0 WIDTH=\"100%%\" cellpadding=0 cellspacing=0>\n");
				printf("<TR>\n");
				printf("<TD ALIGN=LEFT>\n");
				printf("<TABLE BORDER=0 cellpadding=0 cellspacing=0>\n");
				printf("<TR>\n");
				printf("<TD align=left valign=center><A HREF=\"%s?type=%d&host=%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",EXTINFO_CGI,DISPLAY_HOST_INFO,url_encode(temp_status->host_name),temp_status->host_name);
				printf("</TR>\n");
				printf("</TABLE>\n");
				printf("</TD>\n");
				printf("<TD align=right valign=center>\n");
				printf("<TABLE BORDER=0 cellpadding=0 cellspacing=0>\n");
				printf("<TR>\n");
				total_comments=number_of_host_comments(temp_host);
				if(total_comments>0)
					printf("<TD ALIGN=center valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=%d HEIGHT=%d ALT=\"This host has %d comment%s associated with it\"></TD>",url_images_path,COMMENT_ICON,STATUS_ICON_WIDTH,STATUS_ICON_HEIGHT,total_comments,(total_comments==1)?"":"s");
				if(temp_hoststatus->notifications_enabled==FALSE){
					printf("<TD ALIGN=center valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=%d HEIGHT=%d ALT=\"Notifications for this host have been temporarily disabled\"></a></TD>",url_images_path,NOTIFICATIONS_DISABLED_ICON,STATUS_ICON_WIDTH,STATUS_ICON_HEIGHT);
			                }
				if(temp_hostextinfo!=NULL && strcmp(temp_hostextinfo->icon_image,""))
					printf("<TD align=center valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=%d HEIGHT=%d ALT=\"%s\"></TD>\n",url_logo_images_path,temp_hostextinfo->icon_image,STATUS_ICON_WIDTH,STATUS_ICON_HEIGHT,temp_hostextinfo->icon_image_alt);
				printf("</TR>\n");
				printf("</TABLE>\n");
				printf("</TD>\n");
				printf("</TR>\n");
				printf("</TABLE>\n");
			        }
			else
				printf("<TD bgcolor=\"%s\">",HTML_BACKGROUND_COLOR);
			printf("</TD>\n");

			printf("<TD>");
			printf("<TABLE BORDER=0 WIDTH=\"100%%\" CELLSPACING=0 CELLPADDING=0>");
			printf("<TR>");
			printf("<TD ALIGN=LEFT>");
			printf("<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0>\n");
			printf("<TR>\n");
			printf("<TD ALIGN=LEFT valign=center><A HREF=\"%s?type=%d&host=%s",EXTINFO_CGI,DISPLAY_SERVICE_INFO,url_encode(temp_status->host_name));
			printf("&service=%s\"><FONT SIZE=-1>%s</FONT></A></TD>",url_encode(temp_status->description),temp_status->description);
			printf("</TR>\n");
			printf("</TABLE>\n");
			printf("</TD>\n");
			printf("<TD ALIGN=RIGHT>\n");
			printf("<TABLE BORDER=0 cellspacing=0 cellpadding=0>\n");
			printf("<TR>\n");
			total_comments=number_of_service_comments(temp_service);
			if(total_comments>0){
				printf("<TD ALIGN=center valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=%d HEIGHT=%d ALT=\"This service has %d comment%s associated with it\"></a></TD>",url_images_path,COMMENT_ICON,STATUS_ICON_WIDTH,STATUS_ICON_HEIGHT,total_comments,(total_comments==1)?"":"s");
			        }
			if(temp_status->enabled==FALSE){
				printf("<TD ALIGN=center valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=%d HEIGHT=%d ALT=\"This service check has been temporarily disabled\"></a></TD>",url_images_path,DISABLED_ICON,STATUS_ICON_WIDTH,STATUS_ICON_HEIGHT);
			        }
			if(temp_status->notifications_enabled==FALSE){
				printf("<TD ALIGN=center valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=%d HEIGHT=%d ALT=\"Notifications for this service have been temporarily disabled\"></a></TD>",url_images_path,NOTIFICATIONS_DISABLED_ICON,STATUS_ICON_WIDTH,STATUS_ICON_HEIGHT);
			        }
			printf("</TR>\n");
			printf("</TABLE>\n");
			printf("</TD>\n");
			printf("</TR>");
			printf("</TABLE>");
			printf("</TD>\n");

			printf("<TD bgcolor=\"%s\"><A HREF=\"%sstatuslevels.html#%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",status_color,url_docs_path,status,status);
			printf("<TD><FONT SIZE=-1>%s</FONT></TD>\n",date_time);
			printf("<TD><FONT SIZE=-1>%d/%d</FONT></TD>\n",temp_status->current_attempt,temp_status->max_attempts);
			printf("<TD><FONT SIZE=-1>%s&nbsp;</FONT></TD>\n",temp_status->information);
			printf("</TR>\n");

			strncpy(last_host,temp_status->host_name,sizeof(last_host));
			last_host[sizeof(last_host)-1]='\x0';
		        }

	       }

	printf("</TABLE>\n");
	printf("</DIV>\n");
	printf("</P>\n");

	/* if user couldn't see anything, print out some helpful info... */
	if(user_has_seen_something==FALSE){

		printf("<p>\n");
		printf("<div align=\"center\">\n");

		if(servicestatus_list!=NULL){
			printf("<strong><font color=red>It appears as though you do not have permission to view information for any of the services you requested...</font></strong><br><br>\n");
			printf("If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
			printf("and check the authorization options in your CGI configuration file.\n");
		        }
		else{
			printf("<strong><font color=red>There doesn't appear to be any service status information in the status log...</font></strong><br><br>\n");
			printf("Make sure that NetSaint is running and that you have specified the location of you status log correctly in the configuration files.\n");
		        }

		printf("</div>\n");
		printf("</p>\n");
	        }

	return;
        }



/* show an overview of hostgroup(s)... */
void show_hostgroup_overviews(void){
	hostgroup *temp_hostgroup=NULL;
	int current_column;
	int user_has_seen_something=FALSE;
	int hostgroup_error=FALSE;
	int found=0;


	printf("<DIV ALIGN=LEFT>\n");
	printf("<FONT SIZE=-2>\n");

	if(host_status_types!=all_host_status_types){
		found=0;
		printf("Displaying services with host status level(s) of:<b>");
		if(host_status_types & HOST_PENDING){
			printf(" PENDING");
			found=1;
		        }
		if(host_status_types & HOST_UP){
			printf("%s UP",(found==1)?",":"");
			found=1;
		        }
		if(host_status_types & HOST_DOWN){
			printf("%s DOWN",(found==1)?",":"");
			found=1;
		        }
		if(host_status_types & HOST_UNREACHABLE)
			printf("%s UNREACHABLE",(found==1)?",":"");
		printf("</b> \n");
	        }
	else
		printf("Displaying services with <b>all</b> host status levels \n");
	printf("<i>and</i> <b>all</b> service status levels...<br>\n");

	printf("</FONT>\n");
	printf("</DIV>\n");


	/* display status overviews for all hostgroups */
	if(show_all_hostgroups==TRUE){

		printf("<P>\n");
		printf("<DIV ALIGN=CENTER>\n");
		printf("<font size=+1>Status Overview For All Host Groups</font>\n");
		printf("</DIV>\n");
		printf("</P>\n");

		printf("<DIV ALIGN=\"center\">\n");
		printf("<TABLE BORDER=0 CELLPADDING=10>\n");

		current_column=1;

		/* loop through all hostgroups... */
		for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){

			/* make sure the user is authorized to view at least one host in this hostgroup */
			if(is_authorized_for_hostgroup(temp_hostgroup,&current_authdata)==FALSE)
				continue;

			if(current_column==1)
				printf("<TR>\n");
			printf("<TD VALIGN=\"top\" ALIGN=\"center\">\n");
				
			show_hostgroup_overview(temp_hostgroup);

			user_has_seen_something=TRUE;

			printf("</TD>\n");
			if(current_column==overview_columns)
				printf("</TR>\n");

			if(current_column<overview_columns)
				current_column++;
			else
				current_column=1;
		        }

		if(current_column!=1){

			for(;current_column<=overview_columns;current_column++)
				printf("<TD></TD>\n");
			printf("</TR>\n");
		        }

		printf("</TABLE>\n");
		printf("</DIV>\n");
	        }

	/* else display overview for just a specific hostgroup */
	else{

		temp_hostgroup=find_hostgroup(hostgroup_name);
		if(temp_hostgroup!=NULL){

			printf("<P>\n");
			printf("<DIV ALIGN=CENTER>\n");
			printf("<font size=+1>Status Overview For Host Group '%s'</font>\n",hostgroup_name);
			printf("</DIV>\n");
			printf("</P>\n");

			printf("<P>\n");
			printf("<DIV ALIGN=CENTER>\n");
			
			if(is_authorized_for_hostgroup(temp_hostgroup,&current_authdata)==TRUE){

				show_hostgroup_overview(temp_hostgroup);
				
				user_has_seen_something=TRUE;
			        }

			printf("</DIV>\n");
			printf("</P>\n");
		        }
		else{
			printf("<center><font color=red><strong>Sorry, but host group '%s' doesn't seem to exist...</strong></font></center>",hostgroup_name);
			hostgroup_error=TRUE;
		        }
	        }

	/* if user couldn't see anything, print out some helpful info... */
	if(user_has_seen_something==FALSE && hostgroup_error==FALSE){

		printf("<p>\n");
		printf("<div align=\"center\">\n");

		if(hoststatus_list!=NULL){
			printf("<strong><font color=red>It appears as though you do not have permission to view information for any of the hosts you requested...</font></strong><br><br>\n");
			printf("If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
			printf("and check the authorization options in your CGI configuration file.\n");
		        }
		else{
			printf("<strong><font color=red>There doesn't appear to be any host status information in the status log...</font></strong><br><br>\n");
			printf("Make sure that NetSaint is running and that you have specified the location of you status log correctly in the configuration files.\n");
		        }

		printf("</div>\n");
		printf("</p>\n");
	        }

	return;
        }



/* shows an overview of a specific hostgroup... */
void show_hostgroup_overview(hostgroup *hstgrp){
	host *temp_host;
	hoststatus *temp_hoststatus=NULL;
	int odd=0;

	/* make sure the user is authorized to view at least one host in this hostgroup */
	if(is_authorized_for_hostgroup(hstgrp,&current_authdata)==FALSE)
		return;

	printf("<table border=0>\n");
	printf("<tr>\n");
	printf("<td valign=center>\n");
	printf("<font size=-1><A HREF=\"%s?hostgroup=%s&style=detail%s\">%s</A></font>",STATUS_CGI,url_encode(hstgrp->group_name),(display_popup==TRUE)?"":"&nopopup",hstgrp->group_alias);
	printf("</td>\n");
	printf("<td valign=center>\n");
	printf("<font size=-1>(<A HREF=\"%s?type=hostgroups#%s\">%s</A>)</font>",CONFIG_CGI,url_encode(hstgrp->group_name),hstgrp->group_name);
	printf("</td>\n");
	printf("</table>\n");

	printf("<table border=1 bgcolor='#EEEEEE'>\n");

	printf("<tr bgcolor='%s'>\n",html_table_bg_color);
	printf("<th align=left><font color='%s' size=-1>Host Name</font></th>\n",html_table_fg_color);
	printf("<th align=left><font color='%s' size=-1>Host Status</font></th>\n",html_table_fg_color);
	printf("<th align=left><font color='%s' size=-1>Service Status Totals</font></th>\n",html_table_fg_color);
	printf("<th align=left><font color='%s' size=-1>Actions</font></th>\n",html_table_fg_color);
	printf("</tr>\n");

	/* find all the hosts that belong to the hostgroup */
	for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){

		/* find the host... */
		temp_host=find_host(temp_hoststatus->host_name);

		/* make sure this host a member of the hostgroup */
		if(!is_member_of_hostgroup(hstgrp,temp_host))
			continue;

		/* make sure the user is authorized to see this host */
		if(is_authorized_for_host(temp_host,&current_authdata)==FALSE)
			continue;

		/* make sure we only display hosts of the specified status levels */
		if(!(host_status_types & temp_hoststatus->status))
			continue;

		if(odd)
			odd=0;
		else
			odd=1;

		show_hostgroup_member_overview(temp_hoststatus,odd);
	        }

	printf("</table>\n");

	return;
        }

 

/* shows a host status overview... */
void show_hostgroup_member_overview(hoststatus *hststatus,int odd){
	char status[MAX_INPUT_BUFFER];
	char status_color[MAX_INPUT_BUFFER];
	hostextinfo *temp_hostextinfo;

	if(hststatus->status==HOST_PENDING){
		strncpy(status,"PENDING",sizeof(status));
		strncpy(status_color,PENDING_COLOR,sizeof(status_color));
	        }
	else if(hststatus->status==HOST_UP){
		strncpy(status,"UP",sizeof(status));
		strncpy(status_color,OK_COLOR,sizeof(status_color));
	        }
	else if(hststatus->status==HOST_DOWN){
		strncpy(status,"DOWN",sizeof(status));
		strncpy(status_color,CRITICAL_COLOR,sizeof(status_color));
	        }
	else if(hststatus->status==HOST_UNREACHABLE){
		strncpy(status,"UNREACHABLE",sizeof(status));
		strncpy(status_color,CRITICAL_COLOR,sizeof(status_color));
	        }

	status[sizeof(status)-1]='\x0';
	status_color[sizeof(status_color)-1]='\x0';

	if(hststatus->status==HOST_UP || hststatus->status==HOST_PENDING)
		printf("<TR bgcolor=%s>\n",(odd)?html_table_bg_odd_color:html_table_bg_even_color);
	else
		printf("<TR bgcolor=%s>\n",CRITICAL_BG_COLOR);


	/* find extended information for this host */
	temp_hostextinfo=find_hostextinfo(hststatus->host_name);

	printf("<TD>\n");

	printf("<TABLE BORDER=0 WIDTH=\"100%%\" cellpadding=0 cellspacing=0>\n");
	printf("<TR>\n");
	printf("<TD align=left valign=center><A HREF=\"%s?type=%d&host=%s\"><FONT SIZE=-1>%s</FONT></A></TD>\n",EXTINFO_CGI,DISPLAY_HOST_INFO,hststatus->host_name,hststatus->host_name);

	if(temp_hostextinfo!=NULL && strcmp(temp_hostextinfo->icon_image,""))
		printf("<TD align=right valign=center><IMG SRC=\"%s%s\" BORDER=0 WIDTH=%d HEIGHT=%d ALT=\"%s\"></TD>\n",url_logo_images_path,temp_hostextinfo->icon_image,STATUS_ICON_WIDTH,STATUS_ICON_HEIGHT,temp_hostextinfo->icon_image_alt);
	printf("</TR>\n");
	printf("</TABLE>\n");
	printf("</TD>\n");

	printf("<td bgcolor=%s><A HREF=\"%sstatuslevels.html\"><font size=-2>%s</font></A></td>\n",status_color,url_docs_path,status);

	printf("<td>\n");
	show_hostgroup_member_service_status_totals(hststatus->host_name);
	printf("</td>\n");

	printf("<td align=right valign=center>");
	printf("<a href=\"%s?host=%s%s\"><img src=\"%s%s\" border=0 alt=\"View Service Details For This Host\"></a>\n",STATUS_CGI,url_encode(hststatus->host_name),(display_popup==TRUE)?"":"&nopopup",url_images_path,STATUS_DETAIL_ICON);
	printf("<a href=\"%s?host=%s\"><img src=\"%s%s\" border=0 alt=\"View Status Map For This Host\"></a>\n",STATUSMAP_CGI,url_encode(hststatus->host_name),url_images_path,STATUSMAP_ICON);
	printf("<a href=\"%s?host=%s\"><img src=\"%s%s\" border=0 alt=\"View 3-D Status Map For This Host\"></a>\n",STATUSWORLD_CGI,url_encode(hststatus->host_name),url_images_path,STATUSWORLD_ICON);
	printf("</td>");

	printf("</TR>\n");

	return;
        }



void show_hostgroup_member_service_status_totals(char *host_name){
	int total_ok=0;
	int total_warning=0;
	int total_unknown=0;
	int total_critical=0;
	int total_pending=0;
	servicestatus *temp_servicestatus;
	service *temp_service;


	/* check all services... */
	for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){

		if(!strcmp(host_name,temp_servicestatus->host_name)){

			/* make sure the user is authorized to see this service... */
			temp_service=find_service(temp_servicestatus->host_name,temp_servicestatus->description);
			if(is_authorized_for_service(temp_service,&current_authdata)==FALSE)
				continue;

			if(temp_servicestatus->status==SERVICE_CRITICAL)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_HOST_DOWN)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_UNREACHABLE)
				total_critical++;
			else if(temp_servicestatus->status==SERVICE_WARNING)
				total_warning++;
			else if(temp_servicestatus->status==SERVICE_UNKNOWN)
				total_unknown++;
			else if(temp_servicestatus->status==SERVICE_RECOVERY)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_OK)
				total_ok++;
			else if(temp_servicestatus->status==SERVICE_PENDING)
				total_pending++;
			else
				total_ok++;
		        }
	        }


	printf("<TABLE BORDER=\"0\" bgcolor=\"#EEEEEE\">\n");

	printf("<TR>\n");
	if(total_ok>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",OK_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">OK</A></font></TD>\n",total_ok,url_docs_path);
	        }
	if(total_warning>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",WARNING_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">WARNING</A></font></TD>\n",total_warning,url_docs_path);
	        }
	if(total_unknown>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",UNKNOWN_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">UNKNOWN</A></font></TD>\n",total_unknown,url_docs_path);
	        }
	if(total_critical>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",CRITICAL_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">CRITICAL</A></font></TD>\n",total_critical,url_docs_path);
	        }
	if(total_pending>0){
		printf("<TD align=\"center\"");
	        printf(" bgcolor=\"%s\"",PENDING_COLOR);
		printf("><font size=-2>%d <A HREF=\"%sstatuslevels.html\">PENDING</A></font></TD>\n",total_pending,url_docs_path);
	        }

	printf("</TR>\n");
	printf("</TABLE>\n");

	return;
        }


